#ifndef _HTMLBOX_
#define _HTMLBOX_

#include <string>
#include <map>

#include "stree.h"

#ifndef DOXYSKIP
using namespace std;
#endif

/// Stores and refactors html code
class HTMLBox{

 public:
  /// Puts InlineHTML into given HTMLBox
  class Putter : public Node::Transformer{
    HTMLBox &htmlBox;
  public:
    /// Constructor
    Putter(HTMLBox &ihtmlBox) : htmlBox(ihtmlBox) {};
    virtual Node* transform(Node* node) const;
  } putter;

  /// Extracts InlineHTML from given HTMLBox
  class Replacer : public Node::Transformer{
    HTMLBox &htmlBox;
  public:
    /// Constructor 
    Replacer(HTMLBox &ihtmlBox) : htmlBox(ihtmlBox) {};
    virtual Node* transform(Node* node) const;
  } replacer;

 private:

  /// Info about collected html 
  class CollectedHTMLInfo{
  public:
    CollectedHTMLInfo() : occurences(0) {}; 
    /// number the string occures
    int occurences;
    /// refactored html
    string html;
  };

  /// Map for storing info about html 
  typedef map<string,CollectedHTMLInfo> CollectedHTMLInfoMap;

  /// Map for storing info about html 
  CollectedHTMLInfoMap collectedHTML;
  
  /// Output dir for include files
  string outDir;

  /// Used names for templates
  int usedNames;

  /// Is code regenerated
  int regenerated;

  /// Generates a name for include file
  string generateIncludeName();

 public:
  /// constructor
  HTMLBox() : putter(*this), replacer(*this), usedNames(0), regenerated(0) {};

  /// Sets outpt directory
  void setOutDir(const string &ioutDir) { outDir = ioutDir; };
  
  /// Puts html into the box
  /*!
    When html appears more than once an include file is generated, and
    the html is changed to include statement.
   */
  void putHTML(const string &html);

  /// Regenerates collected HTML
  void regenerate();

  /// Returns refactored html.
  string getHTML(const string &html);
  
  /// Normalizes inserted html
  string normalize(const string &html);
};


#endif

