#include "multicontext.h"

MultiContext::MultiContext(RuleContext* isuperContext, const int icontrolVariableId, 
			   const string icontrolVariableName) 
  : RuleContext(isuperContext->rules, isuperContext->nameSpace), 
    superContext(isuperContext), usedNames(0),
    namePrefix(isuperContext->generateVariableName()),
    controlVariableName(icontrolVariableName.substr(1)),
    controlVariableId(icontrolVariableId){
};

MultiContext::MultiContext(RuleContext* isuperContext, const int icontrolVariableId) 
  : RuleContext(isuperContext->rules, isuperContext->nameSpace), 
    superContext(isuperContext), usedNames(0),
    namePrefix(isuperContext->generateVariableName()),
    controlVariableName(generateVariableName()),
    controlVariableId(icontrolVariableId){
  //  cout << "XXX " << controlVariableName << endl;
};
  
const string MultiContext::getVariableNameSufix(const bool stripControlVariableDollar){
    stringstream str;
    str << superContext->getVariableNameSufix(stripControlVariableDollar) << "[";
    if (!stripControlVariableDollar) str << "$";
    str << controlVariableName << "]";
    return str.str();
};

const string MultiContext::getVariableNamePrefix(){
    stringstream str;
    str << namePrefix <<  "_context";
    return str.str();
};


void MultiContext::generateVariableNames(const Node::VariableList &vlist, Node::Assignment &phpAssign, Node::Assignment &tplAssign){

    Node::VariableList::const_iterator it = vlist.begin();
    Node::VariableList::const_iterator eit = vlist.end();
    while (it!=eit){
      if (*it == 0){
	phpAssign.insert(pair<int,const Node*>(0,new Constant(namePrefix + superContext->getVariableNameSufix(false))));
	tplAssign.insert(pair<int,const Node*>(0,new Constant(namePrefix + superContext->getVariableNameSufix(false))));
      } else if (*it == controlVariableId) {
	phpAssign.insert(pair<int,const Node*>(*it, new Constant("$" + controlVariableName)));
	tplAssign.insert(pair<int,const Node*>(*it, new Constant(controlVariableName)));
      } else {
	string newVariableName = generateVariableName();
	phpAssign.insert(pair<int,const Node*>(*it,new Constant(newVariableName + getVariableNameSufix(false))));
	tplAssign.insert(pair<int,const Node*>(*it,new Constant(newVariableName + getVariableNameSufix(false))));
      }
      it++;
    };
};

void MultiContext::enterContext(List *phpOut, List *tplOut) {
  stringstream str;
  //str << "$" << namePrefix << getVariableNameSufix(false) << "=" << "$" << controlVariableName << ";" << endl;
  //    phpOut->push_back(new Constant(str.str()));
  registerSmartyVariable(namePrefix);
};

  
const string MultiContext::generateVariableName(){
    usedNames++;
    stringstream str;
    str << getVariableNamePrefix() << usedNames;
    registerSmartyVariable(str.str());
    return str.str();
};

void MultiContext::registerSmartyVariable(const string &variable){
    superContext->registerSmartyVariable(variable);
};  

void MultiContext::processSmartyAssign(SmartyAssign *smartyAssign){
    stringstream str;
    str << "$" << smartyAssign->getVariable() << "="
	<< smartyAssign->getAssignment() << ";" << endl;
    smartyAssign->setPHPOut(str.str());
};

void MultiContext::processTemplate(Node* tpl) const {
    Node::Assignment newAssign;
    newAssign[controlVariableId] = new Constant(controlVariableName);
    substituteNode(tpl,newAssign);
    
};
