#ifndef _MULTICONTEXT_H_
#define _MULTICONTEXT_H_
#include "rulecontext.h"

/// Context for statements that are executed many times e.g. in for.
/*!
  In this context the valuses of exported variables have to be stored in 
  arrays, with appropriate names. The arrays are indexed by given control variable.
  The names of these arrays are registered.
 */
class MultiContext : public RuleContext{  
  /// A pointer to super context
  RuleContext* superContext;
  /// Number of generated variable names
  int usedNames;
  /// A name prefix for the variables
  const string namePrefix;
  /// The name of the control variable.
  const string controlVariableName;
  /// The id of the control variable.
  const int controlVariableId;

 public:
  /// Constructor 
  MultiContext(RuleContext* isuperContext, const int icontrolVariableId, const string icontrolVariableName);

  /// Constructor, that generates name for a control varaible
  MultiContext(RuleContext* isuperContext, const int icontrolVariableId);

  virtual const string getVariableNameSufix(const bool stripControlVariableDollar);

  virtual const string getVariableNamePrefix();


  virtual void generateVariableNames(const Node::VariableList &vlist, Node::Assignment &phpAssign, Node::Assignment &tplAssign);

  virtual void enterContext(List *phpOut, List *tplOut) ;

  virtual const string generateVariableName();

  virtual void registerSmartyVariable(const string &variable);  

  virtual void processSmartyAssign(SmartyAssign *smartyAssign);

  virtual void processTemplate(Node* tpl) const;
};

#endif
