#include "namedcontext.h"

NamedContext::NamedContext(const string &iname, RuleList &irules, NameSpace &inameSpace) 
  : RuleContext(irules, inameSpace), name(iname), usedNames(0) {
}; 

/*!
  Calls for every variable generateVaraibleName and stores the result in
  \a phpAssign and \tplAssign.
*/
void NamedContext::generateVariableNames(const Node::VariableList &vlist, Node::Assignment &phpAssign, Node::Assignment &tplAssign){
    Node::VariableList::const_iterator it = vlist.begin();
    Node::VariableList::const_iterator eit = vlist.end();
    while (it!=eit){
      string newVariableName = generateVariableName();
      phpAssign.insert(pair<int,const Node*>(*it,new Constant(newVariableName)));
      tplAssign.insert(pair<int,const Node*>(*it,new Constant(newVariableName)));
      it++;
    };
};

const string NamedContext::getVariableNamePrefix(){
    return name;
};


const string NamedContext::getVariableNameSufix(const bool stripControlVariableDollar){
    return "";
};


const string NamedContext::generateVariableName(){
    usedNames++;
    stringstream str;
    str << getVariableNamePrefix() << usedNames;
    registerSmartyVariable(str.str());
    return str.str();
};  

void NamedContext::registerSmartyVariable(const string &variable){
    smartyVariableList.insert(variable);
};

void NamedContext::processSmartyAssign(SmartyAssign *smartyAssign){
    stringstream str;
    str << "$smarty->assign(\"" << smartyAssign->getVariable() 
	<< "\"," << smartyAssign->getAssignment() << ");" << endl;
    smartyAssign->setPHPOut(str.str());
};

void NamedContext::leaveContext(List *phpOut, List *tplOut){
    SmartyVariableList::iterator it = smartyVariableList.begin();
    SmartyVariableList::iterator eit = smartyVariableList.end();
    while (it!=eit){
      stringstream str;
      str << "$smarty->assign_by_ref(\"" << (*it) << "\",$" << (*it) << ");" << endl;
      phpOut->push_back(new Constant(str.str()));
      it++;
    };
};

void NamedContext::processTemplate(Node* tpl) const{
  //  throw "ERROR: NamedContext::processTemplate called";
};



