#include "namespace.h"
#include "config.h"

NameSpace::NameSpace() : superNameSpace(NULL), ruleRunTime(0) {};
NameSpace::NameSpace(NameSpace *isuperNameSpace) : superNameSpace(isuperNameSpace), ruleRunTime(0) {};

void NameSpace::ruleRunTimeStep(){
  ruleRunTime++;
};

int NameSpace::getRuleRunTime() const{
  if (superNameSpace==NULL)
    return ruleRunTime;
  else
    return superNameSpace->getRuleRunTime();
};

void NameSpace::setRuleRunTime(int iruleRunTime){
  if (iruleRunTime<=ruleRunTime)
    throw "ERROR: NameSpace::setRuleRunTime - rule run time was not incremented";
  ruleRunTime = iruleRunTime;
};
/*
void NameSpace::timeStart(){
  time = 0;
};
void NameSpace::timeStep(){
  time++;
};
*/
bool NameSpace::isAttributeSet(const string &name, const string &attribute, const bool global) const{
  if (verbose>5) cout << "NameSpace::isAttributeSet " << name << " " << attribute << " " << global << endl;
  NameAttributesMap::const_iterator found = info.find(name);
  if (((found == info.end())||(global))&&(superNameSpace!=NULL))
    return superNameSpace->isAttributeSet(name,attribute,global);
  else if (found != info.end()) {
    return found->second.find(attribute) != found->second.end();
  }
  else 
    return false;
};

bool NameSpace::isAttributeEqual(const string &name, const string &attribute, const string& value, const bool global) const{
  return getAttributeValue(name,attribute,global) == value;
};

string NameSpace::getAttributeValue(const string &name, const string &attribute, const bool global) const{

  NameAttributesMap::const_iterator found = info.find(name);
  if (((found == info.end())||(global))&&(superNameSpace!=NULL))
    return superNameSpace->getAttributeValue(name,attribute,global);
  else if (found != info.end()) {
    Attributes::const_iterator foundAttribute = found->second.find(attribute);
    if (foundAttribute != found->second.end())
      return foundAttribute->second;
    else
      return "";
  } 
  else
    return "";
};

void NameSpace::setAttributeValue(const string &name, const string &attribute, const string& value, const bool global){
  if (verbose > 5) cout << "NameSpace::setAttributeValue " << name << " " << attribute << " " << value << " " 
			    << global << " " << (superNameSpace!=NULL) << endl;
  if ((global)&&(superNameSpace!=NULL))
    superNameSpace->setAttributeValue(name, attribute, value, global);
  else
    info[name][attribute] = value;
}

void NameSpace::setAttribute(const string &name, const string &attribute, const bool global){
  setAttributeValue(name,attribute,ATTRIBUTETRUE,global);
};

void NameSpace::processArguments(const Node* arguments){
  // TODO: tresc procedury
};

void NameSpace::importLocals(const string& name, const NameSpace &nameSpace){
  NameAttributesMap::const_iterator found = nameSpace.info.find("");
  if (found != nameSpace.info.end()){
    info[name] = found->second;
  }
};
