#ifndef _NAMESPACE_H_
#define _NAMESPACE_H_

#include "streenode.h"
#include <list>
#include <set>

#define ATTRIBUTETRUE "YES"

/// Class for storing names and attributes for the names. 
class NameSpace{
 public:
  /// Maps attributes to values.
  typedef map<const string, string> Attributes;
  /// Maps names to attributes
  typedef map<const string, Attributes> NameAttributesMap;
 private:
  
  /// A super name space.
  NameSpace *superNameSpace;
  /// A map of names and attributes. 
  NameAttributesMap info;
  /// Stores the rule run time.
  int ruleRunTime;

 public:
  /// Constructor.
  NameSpace();
  /// Constructor.
  NameSpace(NameSpace *isuperNameSpace);

  /// Incresases the ruleRunTime by one
  void ruleRunTimeStep(); 
  /// Returns the ruleRunTime
  int getRuleRunTime() const; 
  /// Sets the ruleRunTime
  void setRuleRunTime(int iruleRunTime);

  /// Checks if given attribute is set
  bool isAttributeSet(const string &name, const string &attribute, const bool global) const;
  /// Checks if givan attribute is equal to a given value. 
  bool isAttributeEqual(const string &name, const string &attribute, const string &value, const bool global) const;
  /// Return the value of the attribute
  string getAttributeValue(const string &name, const string &attribute, const bool global) const;
  /// Sets the attribute
  void setAttribute(const string &name, const string &attribute, const bool global);
  /// Sets the value of the attribute
  void setAttributeValue(const string &name, const string &attribute, const string &value, const bool global);
  /// Processes arguments -- called by NamedContextRule -- now empty no assumptions made on arguments. 
  void processArguments(const Node* arguments);
  /// Imports attributes for an empty name from nameSpace as an attributes for a given name 
  void importLocals(const string& name, const NameSpace &nameSpace);
};

#endif
