%{
#include "stree.h"
#include "streeconstant.h"
#include "streevariable.h"
#include "streelist.h"
#include "streepin.h"
#include "rtree.h"
#include "rules.h"
#include "ctree.h"
#include "rulesystem.h"
#include "testers.h"
#include "treeparser.h"
%}
%union {
  Constant* constant_val;

  Variable* phpx_variable_val;
  Node* phpx_context_variable_val;

  List* statement_list_val;
  List* template_list_val;
  Node* template_val;

  Node* statement_val;
  Node* if_val;
  Node* expr_val;
  
  Node* var_list_val;

  Node* expr_list_val;
  Node* declare_list_val;
  Node* case_list_val;
  Node* case_val;

  Node* array_pair_list_val;
  Node* encaps_list_val;

  StatementList* main_list_val;
  List* inlinehtml_list_val;

  RuleList* rule_list_val;
  PriorityRulePair* priority_rule_pair_val;

  Node::Tester* tester_val;
  TesterSet* tester_set_val;

  Action* action_val;
  ActionList* action_list_val;
  
  Node* action_set_name_val;
}

%{
  RuleSystem* factor;

  extern ParseType parseRules;

  int yylex();
  void yyerror(const char *); 

  void *yymyresult;

#define YYDEBUG 1
#define DEBUG 1
#define YYERROR_VERBOSE

%}

%expect 6

%left T_INCLUDE T_INCLUDE_ONCE T_EVAL T_REQUIRE T_REQUIRE_ONCE
%left ','
%left T_LOGICAL_OR
%left T_LOGICAL_XOR
%left T_LOGICAL_AND
%right T_PRINT
%left '=' T_PLUS_EQUAL T_MINUS_EQUAL T_MUL_EQUAL T_DIV_EQUAL T_CONCAT_EQUAL T_MOD_EQUAL T_AND_EQUAL T_OR_EQUAL T_XOR_EQUAL T_SL_EQUAL T_SR_EQUAL
%left '?' ':'
%left T_BOOLEAN_OR
%left T_BOOLEAN_AND
%left '|'
%left '^'
%left '&'
%nonassoc T_IS_EQUAL T_IS_NOT_EQUAL T_IS_IDENTICAL T_IS_NOT_IDENTICAL
%nonassoc '<' T_IS_SMALLER_OR_EQUAL '>' T_IS_GREATER_OR_EQUAL
%left T_SL T_SR
%left '+' '-' '.'
%left '*' '/' '%'
%right '!' '~' T_INC T_DEC T_INT_CAST T_DOUBLE_CAST T_STRING_CAST T_ARRAY_CAST T_OBJECT_CAST T_BOOL_CAST T_UNSET_CAST '@'
%right '['
%nonassoc T_NEW
%token T_EXIT
%token T_IF
%left T_ELSEIF
%left T_ELSE
%left T_ENDIF
%token<constant_val> T_LNUMBER
%token<constant_val> T_DNUMBER
%token<constant_val> T_STRING
%token<constant_val> T_STRING_VARNAME
%token<constant_val> T_VARIABLE
%token<constant_val> T_NUM_STRING
%token<constant_val> T_INLINE_HTML
%token<constant_val> T_CHARACTER
%token<constant_val> T_BAD_CHARACTER
%token<constant_val> T_ENCAPSED_AND_WHITESPACE
%token<constant_val> T_CONSTANT_ENCAPSED_STRING
%token<phpx_variable_val> T_CONSTANT_ENCAPSED_PHPX_VARIABLE
%token<phpx_variable_val> T_PHPX_VARIABLE
%token<phpx_variable_val> T_PHPX_LIST_VARIABLE
%token T_ECHO
%token T_TEMPLATE
%token T_DO
%token T_WHILE
%token T_ENDWHILE
%token T_FOR
%token T_ENDFOR
%token T_FOREACH
%token T_ENDFOREACH
%token T_DECLARE
%token T_ENDDECLARE
%token T_AS
%token T_SWITCH
%token T_ENDSWITCH
%token T_CASE
%token T_DEFAULT
%token T_BREAK
%token T_CONTINUE
%token T_OLD_FUNCTION
%token T_FUNCTION
%token T_CONST
%token T_RETURN
%token T_USE
%token T_GLOBAL
%token T_STATIC
%token T_VAR
%token T_UNSET
%token T_ISSET
%token T_EMPTY
%token T_CLASS
%token T_EXTENDS
%token T_OBJECT_OPERATOR
%token T_DOUBLE_ARROW
%token T_LIST
%token T_ARRAY
%token<constant_val> T_CLASS_C
%token<constant_val> T_FUNC_C
%token<constant_val> T_LINE
%token<constant_val> T_FILE
%token T_COMMENT
%token T_ML_COMMENT
%token T_OPEN_TAG
%token T_RULE_OPEN_TAG
%token T_NORMALIZER_OPEN_TAG
%token T_OPEN_TAG_WITH_ECHO
%token T_CLOSE_TAG
%token T_WHITESPACE
%token T_START_HEREDOC
%token T_END_HEREDOC
%token T_DOLLAR_OPEN_CURLY_BRACES
%token T_CURLY_OPEN
%token T_PAAMAYIM_NEKUDOTAYIM

%token T_SUBSTITUTION_RULE
%token T_SINGLE_CONTEXT_RULE
%token T_MULTI_CONTEXT_RULE
%token T_NAMED_CONTEXT_RULE
%token T_TRIGGERED_RULE
%token T_INCLUDE_RULE

%token T_NORMALIZE
%token T_SMARTY_ASSIGN

%token T_NAME
%token T_EXISTS
%token T_FORALL
%token T_NOT
%token T_FORALL_TESTERS
%token T_EXISTS_TESTER
%token T_ACTION_RULE
%token T_WRITE
%token T_SET
%token T_ISEQUAL
%token T_TISSET
%token T_VALUE
%token T_GLOBAL_VALUE
%token T_TYPE

//%type <main_list_val> html_or_script_list
%type <statement_list_val> html_or_script


%type <statement_list_val> top_statement_list
%type <statement_list_val> context_top_statement_list
%type <statement_val> top_statement
%type <statement_list_val> start
%type <statement_val> statement

%type <statement_val> declaration_statement
%type <statement_list_val> inner_statement_list;
%type <statement_val> inner_statement;

%type <statement_val> unticked_statement
%type <statement_val> while_statement

%type <if_val> elseif_list 

%type <statement_val> for_statement

%type <expr_val> expr
%type <expr_val> exit_expr


%type <var_list_val> static_var_list
%type <var_list_val> global_var_list

%type <expr_list_val> echo_expr_list
%type <expr_list_val> for_expr
%type <expr_val> use_filename
%type <statement_val> w_cvar

%type <statement_val> foreach_optional_arg
%type <statement_val> foreach_statement 
%type <expr_val> expr_without_variable

%type <declare_list_val> declare_list
%type <statement_val> declare_statement 

%type <statement_val> unset_variable
%type <var_list_val> unset_variables

%type <statement_val> cvar

%type <statement_val> unticked_declaration_statement

%type <statement_val> is_reference 
%type <declare_list_val> parameter_list 

%type <statement_list_val> class_statement_list

%type <case_list_val> switch_case_list
%type <case_list_val> case_list

%type <statement_val> static_scalar

%type <declare_list_val> non_empty_parameter_list

%type <declare_list_val> function_call_parameter_list
%type <declare_list_val> non_empty_function_call_parameter_list

%type <statement_val> global_var

%type <statement_val> r_cvar
%type <statement_val> rw_cvar

%type <declare_list_val> class_variable_decleration
%type <statement_val> class_statement

%type <expr_list_val> non_empty_for_expr

%type <expr_list_val> assignment_list
%type <statement_val> assignment_list_element


%type <statement_val> function_call
%type <statement_val> static_or_variable_string
%type <statement_val> scalar

%type <array_pair_list_val> array_pair_list
%type <array_pair_list_val> static_array_pair_list
%type <array_pair_list_val> non_empty_static_array_pair_list
%type <array_pair_list_val> non_empty_array_pair_list

%type <encaps_list_val> encaps_list

%type <expr_val> common_scalar

%type <statement_val> cvar_without_objects
%type <statement_val> ref_list
%type <statement_val> reference_variable

%type <statement_val> dim_offset 

%type <statement_val> compound_variable
%type <statement_val> object_property

%type <statement_val> object_dim_list
%type <statement_val> variable_name

%type <statement_val> encaps_var
%type <constant_val> encaps_var_offset

%type <rule_list_val> rule_list
%type <priority_rule_pair_val> rule

%type <template_list_val> template_list
%type <template_list_val> non_empty_template_list
%type <template_val> template

%type <phpx_context_variable_val> phpx_context_variable
%type <phpx_context_variable_val> phpx_context_list_variable

%type <statement_list_val> optional_html
%type <inlinehtml_list_val> inline_html

%type <tester_val> tester
%type <tester_set_val> tester_exists_list
%type <tester_set_val> tester_forall_list
%type <tester_val> local_tester
%type <tester_set_val> local_tester_exists_list
%type <tester_set_val> local_tester_forall_list

%type <action_val> action
%type <action_list_val> action_list

%type <action_set_name_val> action_set_name
%type <constant_val> attribute_name

%% /* Rules */

start:
		html_or_script				  { if (parseRules == ParsePHP ) yymyresult = $1; 
							    else throw "Error: Parsed PHP" }
	|	T_RULE_OPEN_TAG T_CONSTANT_ENCAPSED_STRING T_LNUMBER rule_list T_CLOSE_TAG optional_html
							  { if(parseRules == ParseRules) yymyresult = new NamedRuleList($2,$3,$4);
							    else throw "Error: Parsed Rules" }
//	|	T_NORMALIZER_OPEN_TAG normalizer_list T_CLOSE_TAG optional_html
//							  { if(parseRules == ParseNormalizers) yymyresult = $2;
//							    else throw "Error: Parsed Normalizers" }
;

optional_html:
		T_INLINE_HTML optional_html 	{ $$ = $2->push_front(new InlineHTML($1)) }
	|					{ $$ = new InlineHTMLList() }
;

html_or_script:
        	optional_html T_OPEN_TAG top_statement_list T_CLOSE_TAG optional_html 		
			      { $$ = $3->push_front($1)->push_back($5) }//$$ = dynamic_cast<StatementList*>($1->append($3)->append($4)) }
	|	optional_html T_OPEN_TAG_WITH_ECHO top_statement_list T_CLOSE_TAG optional_html  
			      { $$ = $3->push_front($1)->push_back($5) }//$$ = dynamic_cast<StatementList*>($1->append($3)->append($4)) }
        |	optional_html { $$ = $1 }
;

top_statement_list:
		top_statement_list top_statement { $$ = $1->push_back($2) }
	|	/* empty */			 { $$ =  new StatementList() }
;

context_top_statement_list:
		top_statement_list				{ $$ = $1 }
	|       phpx_context_list_variable			{ $$ = (List*)($1) }
	|	top_statement_list ':' local_tester		{ $$ = (List*) (new ContextMatch($1,$3)) }
	|       phpx_context_list_variable ':' local_tester	{ $$ = (List*) (new ContextMatch($1,$3)) } 
;


top_statement:
		statement { $$=$1; }
	|	declaration_statement	{ $$=$1 }
;


inner_statement_list:
		inner_statement_list  inner_statement      { $$ = $1->push_back($2) }
        |       phpx_context_list_variable                       { $$ = (List*)($1) }
	|	/* empty */ { $$ = new StatementList() }
;

inner_statement:
		statement { $$ = $1 }
	|	declaration_statement { $$ = $1 } 
;


statement:
		unticked_statement                  { $$ = $1 } //new CurlyBraces(new StatementList($1)) }	
	|      	'{' inner_statement_list '}' { $$ = new CurlyBraces($2) }

;

local_tester:
		T_TISSET '{' T_VALUE attribute_name '}'			{ $$ = new LocalAttributeIssetTester($4, false) }
	|	T_ISEQUAL '{' T_VALUE attribute_name '=' attribute_name '}' { $$ = new LocalAttributeIsequalTester($4,$6, false) } 
	|	T_TISSET '{' T_GLOBAL_VALUE attribute_name '}'			{ $$ = new LocalAttributeIssetTester($4, true) }
	|	T_ISEQUAL '{' T_GLOBAL_VALUE attribute_name '=' attribute_name '}' { $$ = new LocalAttributeIsequalTester($4,$6, true) } 
	|	T_NOT     '{' local_tester '}'			{ $$ = new NotTester($3) }
	|	T_FORALL_TESTERS '{' local_tester_forall_list '}'		{ $$ = $3 }
	|	T_EXISTS_TESTER '{' local_tester_exists_list '}'		{ $$ = $3 }
;

local_tester_exists_list:
		local_tester_exists_list ',' local_tester { $$ = $1->insert($3) }
	|	local_tester		              { $$ = new TesterSetExists($1) }
;
local_tester_forall_list:
		local_tester_forall_list ',' local_tester { $$ = $1->insert($3) }
	|	local_tester		              { $$ = new TesterSetForall($1) }
;



rule_list:
		rule_list rule { $1->insert($2) }
	|	/* empty */    { $$ = new RuleList() }
;

rule:
		T_SUBSTITUTION_RULE '(' T_LNUMBER '|' context_top_statement_list '|'
					template_list '|' top_statement_list ')' ';'
					{  $$ = new PriorityRulePair($3,new SubstitutionRule($5,$7,$9)) }
	|	T_SUBSTITUTION_RULE '(' T_LNUMBER '|' context_top_statement_list '|'
					template_list '|' top_statement_list '|' action_list ')' ';'
					{  $$ = new PriorityRulePair($3,
					  new RuleWithActionRule(new SubstitutionRule($5,$7,$9), $11)) }
        |       T_SINGLE_CONTEXT_RULE '(' T_LNUMBER '|' context_top_statement_list '|' 
					  template_list '|' top_statement_list ')' ';'
					  { $$ = new PriorityRulePair($3,new SingleContextRule($5,$7,$9)) }
        |       T_SINGLE_CONTEXT_RULE '(' T_LNUMBER '|' context_top_statement_list '|' 
					  template_list '|' top_statement_list '|' action_list ')' ';'
					  { $$ = new PriorityRulePair($3,new RuleWithActionRule(
					    new SingleContextRule($5,$7,$9), $11)) }
        |       T_MULTI_CONTEXT_RULE '(' T_LNUMBER '|' context_top_statement_list '|' T_PHPX_VARIABLE '|'
 					 template_list '|' top_statement_list ')' ';'
	 				 { $$ = new PriorityRulePair($3,new MultiContextRule($5,$7,$9,$11)) }
        |       T_MULTI_CONTEXT_RULE '(' T_LNUMBER '|' context_top_statement_list '|' T_PHPX_VARIABLE '|'
 					 template_list '|' top_statement_list '|' action_list ')' ';'
	 				 { $$ = new PriorityRulePair($3,new RuleWithActionRule(
					   new MultiContextRule($5,$7,$9,$11),$13)) }
	|       T_TRIGGERED_RULE '(' T_LNUMBER '|' rule '|' rule ')' ';'
				         { $$ = new PriorityRulePair($3,new TriggeredRule($5,$7)) }
	|       T_TRIGGERED_RULE '(' T_LNUMBER '|' rule '|' rule '|' action_list ')' ';'
				         { $$ = new PriorityRulePair($3,new RuleWithActionRule(new TriggeredRule($5,$7),$9)) }
	|       T_NAMED_CONTEXT_RULE '(' T_LNUMBER '|' T_CONSTANT_ENCAPSED_STRING '|' context_top_statement_list '|' top_statement_list '|'
					 T_PHPX_VARIABLE '|' T_PHPX_VARIABLE '|' T_PHPX_VARIABLE  ')' ';'
				         { $$ = new PriorityRulePair($3,new NamedContextRule(
					        $5,factor->getRunRuleList($5->getValue()),$7,$9,$11,$13,$15)) }
	|       T_NAMED_CONTEXT_RULE '(' T_LNUMBER '|' T_CONSTANT_ENCAPSED_STRING '|' context_top_statement_list '|' top_statement_list '|'
					 T_PHPX_VARIABLE '|' T_PHPX_VARIABLE '|' T_PHPX_VARIABLE '|' action_list ')' ';'
				         { $$ = new PriorityRulePair($3,new RuleWithActionRule(new NamedContextRule(
					        $5,factor->getRunRuleList($5->getValue()),$7,$9,$11,$13,$15), $17)) }
	|	T_ACTION_RULE '(' T_LNUMBER '|' context_top_statement_list '|' action_list ')' ';'
					 { $$ = new PriorityRulePair($3,new ActionRule($5,$7)) }
	|	T_INCLUDE_RULE  '(' T_LNUMBER '|' T_CONSTANT_ENCAPSED_STRING ')' ';'
			                 { $$ = new PriorityRulePair($3,new IncludeRule($5,factor->getRunRuleList($5->getValue()))) }

	|	T_INCLUDE_RULE  '(' T_LNUMBER '|' T_CONSTANT_ENCAPSED_STRING '|' action_list ')' ';'
			                 { $$ = new PriorityRulePair($3,new RuleWithActionRule(
					   new IncludeRule($5,factor->getRunRuleList($5->getValue())),$7)) }
;

action_list:
		action_list ',' action	   { $$ = $1->push_back($3) }
	|	action			   { $$ = new ActionList($1) }
;

action:
		T_WRITE	expr		   { $$ = new ActionWrite($2) }
	|	T_SET action_set_name T_VALUE attribute_name { $$ = new ActionSet($2,$4,new Constant(ATTRIBUTETRUE),false) }
	|	T_SET action_set_name T_VALUE attribute_name ':' '=' expr { $$ = new ActionSet($2,$4,$7,false) }
	|	T_SET T_VALUE attribute_name { $$ = new ActionSet(new Pin(),$3,new Constant(ATTRIBUTETRUE),false) }
	|	T_SET T_VALUE  attribute_name ':' '=' expr { $$ = new ActionSet(new Pin(),$3,$6,false) }
	|	T_SET action_set_name T_GLOBAL_VALUE attribute_name { $$ = new ActionSet($2,$4,new Constant(ATTRIBUTETRUE), true) }
	|	T_SET action_set_name T_GLOBAL_VALUE attribute_name ':' '=' expr { $$ = new ActionSet($2,$4,$7,true) }
	|	T_SET T_GLOBAL_VALUE attribute_name { $$ = new ActionSet(new Pin(),$3,new Constant(ATTRIBUTETRUE),true) }
	|	T_SET T_GLOBAL_VALUE  attribute_name ':' '=' expr { $$ = new ActionSet(new Pin(),$3,$6,true) }
;

action_set_name:
		T_PHPX_VARIABLE			{ $$ = $1 }
	|	attribute_name			{ $$ = $1 }
;

attribute_name:
		T_STRING			{ $$ = $1 }
	|	T_CONSTANT_ENCAPSED_STRING	{ $$ = $1 }
	|	T_LNUMBER			{ $$ = $1 }
;		


template_list:
		non_empty_template_list    { $$ = $1 }
	|	/* empty */		   { $$ = new TemplateList() }
;

non_empty_template_list:
		template_list ',' template { $$ = $1->push_back($3) }
	|	template	           { $$ = new TemplateList($1) }
;

template:
		T_PHPX_VARIABLE			{ $$ = $1 }
	|	T_PHPX_LIST_VARIABLE		{ $$ = $1 }
	|       T_TEMPLATE '"' encaps_list '"'  { $$ = new Template($3) }
;

phpx_context_variable:
		T_PHPX_VARIABLE					{ $$ = $1 }
	|       phpx_context_variable tester			{ $$ = new ContextMatch($1,$2) }
;

phpx_context_list_variable:
		T_PHPX_LIST_VARIABLE					{ $$ = $1 }
	|       phpx_context_list_variable tester			{ $$ = new ContextMatch($1,$2) }
;

tester:
		T_NAME      '{' action_set_name '}'			{ $$ = new NameTester($3) }
	|	T_TYPE      '{' action_set_name '}'			{ $$ = new TypeTester($3) }
	|	T_EXISTS    '{' tester '}'			{ $$ = new ExistsTester($3) }
	|	T_FORALL    '{' tester '}'			{ $$ = new ForallTester($3) }
	|	T_EXISTS    '{' tester ':' tester '}'			{ $$ = new ExistsTester(new TesterSetExists(new NotTester($3),$5)) }
	|	T_FORALL    '{' tester ':' tester '}'			{ $$ = new ForallTester(new TesterSetExists(new NotTester($3),$5)) }
	|	T_NOT       '{' tester '}'			{ $$ = new NotTester($3) }
	|	T_TISSET '{' T_VALUE attribute_name '}'			{ $$ = new AttributeIssetTester($4, false) }
	|	T_ISEQUAL '{' T_VALUE attribute_name '=' attribute_name '}'	{ $$ = new AttributeIsequalTester($4,$6, false) }
	|	T_TISSET '{' T_GLOBAL_VALUE attribute_name '}'			{ $$ = new AttributeIssetTester($4, true) }
	|	T_ISEQUAL '{' T_GLOBAL_VALUE attribute_name '=' attribute_name '}'	{ $$ = new AttributeIsequalTester($4,$6, true) }
	|	T_FORALL_TESTERS '{' tester_forall_list '}'		{ $$ = $3 }
	|	T_EXISTS_TESTER '{' tester_exists_list '}'		{ $$ = $3 }
;

tester_exists_list:
		tester_exists_list ',' tester { $$ = $1->insert($3) }
	|	tester		              { $$ = new TesterSetExists($1) }
;
tester_forall_list:
		tester_forall_list ',' tester { $$ = $1->insert($3) }
	|	tester		              { $$ = new TesterSetForall($1) }
;


inline_html:
		T_INLINE_HTML inline_html  { $$ = $2->push_front($1) }
	|	T_INLINE_HTML		   { $$ = new InlineHTMLList($1) }
;

unticked_statement:
	 	T_DO statement T_WHILE '(' expr ')' ';' { $$ = new Do($2,$5) }
	|	T_BREAK ';'				{ $$ = new Break(new Pin()) }
	|	T_BREAK expr ';'		{ $$ = new Break($2) }
	|	T_CONTINUE ';'			{ $$ = new Continue(new Pin()) }
	|	T_CONTINUE expr ';'		{ $$ = new Continue($2) }
	|	T_RETURN ';'			{ $$ = new Return(new Pin()) }
	|	T_RETURN expr ';'		{ $$ = new Return($2) }
        |	T_GLOBAL global_var_list ';'    { $$ = new Global($2) }
        |	T_STATIC static_var_list ';'    { $$ = new Static($2) }
	|	T_ECHO echo_expr_list ';'       { $$ = new Echo($2) }
	|	T_SMARTY_ASSIGN '(' expr ',' expr ')' ';' { $$ = new SmartyAssign($3,$5) }
	|	expr ';'			{ if ($1->getType() != SVariable) $$ = new Expression($1); else $$=$1}
	|	T_USE use_filename ';'		{ $$ = new Use($2)  }
	|	T_UNSET '(' unset_variables ')' ';' { $$ = new Unset($3) }
	|	T_IF '(' expr ')' statement elseif_list { $$ = new If($3,$5,$6) }
     	|       T_CLOSE_TAG inline_html T_OPEN_TAG  { $$ = $2 }
	|	T_WHILE '(' expr  ')' while_statement { $$ = new While($3,$5) }
	|	T_FOR '(' for_expr ';' for_expr ';' for_expr ')' for_statement 
		      { $$ = new For($3,$5,$7,$9) }
	|	T_SWITCH '(' expr ')'  switch_case_list { $$ = new Switch($3,$5) }
	|	T_FOREACH '(' w_cvar T_AS w_cvar foreach_optional_arg ')' foreach_statement 
			  { $$ = new Foreach($3,$5,$6,$8) } 
	|	T_FOREACH '(' expr_without_variable T_AS w_cvar foreach_optional_arg ')' foreach_statement 
                          { $$ = new Foreach($3,$5,$6,$8) }
	|	T_DECLARE '(' declare_list ')' declare_statement { $$ = new Declare($3,$5) }
	|	';'		/* empty statement */ { $$ = new EmptyStatement() }
;


unset_variables:
	        unset_variable { $$ = new ExpressionList($1) }
	|	unset_variables ',' unset_variable { $$ = $1->push_back($3) }
;

unset_variable:
		cvar	{ $$=$1 }
;

use_filename:
		T_CONSTANT_ENCAPSED_STRING		{ $$ = $1 }
	|	'(' T_CONSTANT_ENCAPSED_STRING ')'	{ $$ = $2 }
;


declaration_statement:
		unticked_declaration_statement	{ $$=$1 }
;


unticked_declaration_statement:
		T_FUNCTION is_reference T_STRING '(' parameter_list ')' '{' inner_statement_list '}' 
			   { $$=new Function($2,$3,$5,$8) }
	|	T_FUNCTION T_PHPX_VARIABLE '(' parameter_list ')' '{' inner_statement_list '}' 
			   { $$=new Function(new Pin(),$2,$4,$7) }
	|	T_OLD_FUNCTION is_reference T_STRING  parameter_list '(' inner_statement_list ')' ';' 
			   { $$=new Function($2,$3,$4,$6) }
	|	T_OLD_FUNCTION T_PHPX_VARIABLE  parameter_list '(' inner_statement_list ')' ';' 
			   { $$=new Function(new Pin(),$2,$3,$5) }
	|	T_CLASS T_STRING '{' class_statement_list '}' 
		           { $$=new Class($2,new Pin(),$4); 
			     cout << "Warning: class refactoring is not yet implemented." <<endl }
	|	T_CLASS T_PHPX_VARIABLE '{' class_statement_list '}' 
			   { $$=new Class($2,new Pin(),$4);	
			     cout << "Warning: class refactoring is not yet implemented." <<endl }
	|	T_CLASS T_STRING T_EXTENDS T_STRING '{' class_statement_list '}' 
			   { $$=new Class($2,$4,$6);
			     cout << "Warning: class refactoring is not yet implemented." <<endl }
	|	T_CLASS T_PHPX_VARIABLE T_EXTENDS T_PHPX_VARIABLE '{' class_statement_list '}' 
			   { $$=new Class($2,$4,$6);
			     cout << "Warning: class refactoring is not yet implemented." <<endl }
;


foreach_optional_arg:
		/* empty */				{ $$= new Pin() }
	|	T_DOUBLE_ARROW w_cvar			{ $$ = $2 }
;


for_statement:
		statement				{ $$=$1 }
	|	':' inner_statement_list T_ENDFOR ';'	{ $$=$2 }
;


foreach_statement:
		statement					{ $$=$1 }
	|	':' inner_statement_list T_ENDFOREACH ';'	{ $$=$2 }
;


declare_statement:
		statement					{ $$=$1 }
	|	':' inner_statement_list T_ENDDECLARE ';'	{ $$=$2 }
;


declare_list:
		T_STRING '=' static_scalar			{ $$=new DeclareList(new Parameter(new Pin(),$1,$3)) }
	|	declare_list ',' T_STRING '=' static_scalar	{ $$=$1->push_back(new Parameter(new Pin(),$3,$5)) }
;


switch_case_list:
		'{' case_list '}'					{ $$=$2 }
	|	'{' ';' case_list '}'					{ $$=$3 }
	|	':' case_list T_ENDSWITCH ';'				{ $$=$2 }
	|	':' ';' case_list T_ENDSWITCH ';'			{ $$=$3 }
;


case_list:
		/* empty */							{ $$ = new CaseList() }
	|	case_list T_CASE expr case_separator inner_statement_list	{ $$ = $1->push_back(new Case($3,$5)) }
	|	case_list T_DEFAULT case_separator inner_statement_list		{ $$ = $1->push_back(new Case(new Pin(),$4)) }
;


case_separator:
		':'
	|	';'
;
	

while_statement:
		statement					{ $$=$1 }
	|	':' inner_statement_list T_ENDWHILE ';'		{ $$=$2 }
;



elseif_list:
		/* empty */					{ $$ = new Pin() }
	|       T_ELSEIF '(' expr ')' statement elseif_list	{ $$ = new If($3,$5,$6) }
	|       T_ELSE statement				{ $$ = $2 }
;


parameter_list: 
		non_empty_parameter_list			{ $$ = $1 }
	|       phpx_context_list_variable				{ $$ = $1 }
	|	/* empty */					{ $$ = new DeclareList() }
;


non_empty_parameter_list:
		T_VARIABLE				{ $$ = new DeclareList(new Parameter(new Pin(),new PHPVariable($1),new Pin())) }
	|	'&' T_VARIABLE				{ $$ = new DeclareList(new Parameter(new Constant("&"),new PHPVariable($2)
												 ,new Pin())) }
	|	T_CONST T_VARIABLE			{ $$ = new DeclareList(new Parameter(new Constant("const"),
												 new PHPVariable($2),new Pin())) }
	|	T_VARIABLE '=' static_scalar		{ $$ = new DeclareList(new Parameter(new Pin(),new PHPVariable($1),$3)) }
	|	phpx_context_variable				{ $$ = new DeclareList($1) }
	|	non_empty_parameter_list ',' T_VARIABLE			   { $$=$1->push_back(new Parameter(new Pin(),new PHPVariable($3),new Pin())) }
	|	non_empty_parameter_list ',' '&' T_VARIABLE		   { $$=$1->push_back(new Parameter(new Constant("&"),new PHPVariable($4),new Pin())) }
	|	non_empty_parameter_list ',' T_CONST T_VARIABLE		   { $$=$1->push_back(new Parameter(new Constant("const"),new PHPVariable($4),new Pin())) }
	|	non_empty_parameter_list ',' T_VARIABLE '=' static_scalar  { $$=$1->push_back(new Parameter(new Pin(),new PHPVariable($3),$5)) }
	|       non_empty_parameter_list ',' phpx_context_variable		   { $$=$1->push_back($3) }
;


function_call_parameter_list:
		non_empty_function_call_parameter_list	{ $$ = $1 }
	|       phpx_context_list_variable			{ $$ = $1 }
	|	/* empty */				{ $$ = new ExpressionList() }
;


non_empty_function_call_parameter_list:
		expr_without_variable			{ $$ = new ExpressionList(new Parameter(new Pin(),$1,new Pin())) }
	|	cvar					{ $$ = new ExpressionList(new Parameter(new Pin(),$1,new Pin())) }
	|	'&' w_cvar				{ $$ = new ExpressionList(new Parameter(new Constant("&"),$2,new Pin())) }
	|	non_empty_function_call_parameter_list ',' expr_without_variable	{ $$=$1->push_back(new Parameter(new Pin(),$3,new Pin())) }
	|	non_empty_function_call_parameter_list ',' cvar				{ $$=$1->push_back(new Parameter(new Pin(),$3,new Pin())) }
	|	non_empty_function_call_parameter_list ',' '&' w_cvar			{ $$=$1->push_back(new Parameter(new Constant("&"),$4,new Pin())) }
;

global_var_list:
		global_var_list ',' global_var		{ $$=$1->push_back($3) }
	|	global_var				{ $$=new DeclareList($1) }
;


global_var:
		T_VARIABLE			{ $$ = new PHPVariable($1) }
	|	'$' r_cvar			{ $$ = new UnaryOperator("$",$2) }
	|	'$' '{' expr '}'		{ $$ = new MapOperator(new Constant("$"),$3) }
;


static_var_list:
		static_var_list ',' T_VARIABLE			 { $$=$1->push_back(new Parameter(new Pin(),new PHPVariable($3),new Pin())) }
	|	static_var_list ',' T_VARIABLE '=' static_scalar { $$=$1->push_back(new Parameter(new Pin(),new PHPVariable($3),$5)) }
	|	T_VARIABLE					 { $$=new DeclareList(new Parameter(new Pin(),new PHPVariable($1),new Pin())) }
	|	T_VARIABLE '=' static_scalar			 { $$=new DeclareList(new Parameter(new Pin(),new PHPVariable($1),$3)) }

;


class_statement_list:
		class_statement_list class_statement	{ $$=$1->push_back($2) }
	|	/* empty */				{ $$=new StatementList() }
;


class_statement:
		T_VAR class_variable_decleration ';'	{ $$=new ClassVariable($2) }
	|	T_FUNCTION is_reference T_STRING '(' parameter_list ')' '{' inner_statement_list '}' { $$= new Function($2,$3,$5,$8) }
	|	T_FUNCTION T_PHPX_VARIABLE '(' parameter_list ')' '{' inner_statement_list '}' { $$= new Function(new Pin(),$2,$4,$7) }
	|	T_OLD_FUNCTION is_reference T_STRING parameter_list '(' inner_statement_list ')' ';' { $$= new Function($2,$3,$4,$6) }
	|	T_OLD_FUNCTION T_PHPX_VARIABLE parameter_list '(' inner_statement_list ')' ';' { $$= new Function(new Pin(),$2,$3,$5) }
;

is_reference:
		/* empty */	{ $$ = new Pin() }
	|	'&'		{ $$ = new Constant("&") }
	|       T_PHPX_VARIABLE { $$ = $1 }
;

class_variable_decleration:
		class_variable_decleration ',' T_VARIABLE			{ $$=$1->push_back(new Parameter(new Pin(),new PHPVariable($3),new Pin())) }
	|	class_variable_decleration ',' T_VARIABLE '=' static_scalar	{ $$=$1->push_back(new Parameter(new Pin(),new PHPVariable($3),$5)) }
	|	T_VARIABLE							{ $$= new DeclareList(new Parameter(new Pin(),new PHPVariable($1),new Pin())) }
	|	T_VARIABLE '=' static_scalar					{ $$= new DeclareList(new Parameter(new Pin(),new PHPVariable($1),$3)) }
;


echo_expr_list:		
		/* empty */			{ $$=new ExpressionList() }
	|	echo_expr_list ',' expr		{ $$=$1->push_back($3) }
	|	expr				{ $$=new ExpressionList($1) }
	|	phpx_context_list_variable		{ $$ = $1 }
;


for_expr:
		/* empty */			{ $$=new ExpressionList() }
	|	non_empty_for_expr		{ $$=$1 }
;

non_empty_for_expr:
		non_empty_for_expr ','	expr	{ $$=$1->push_back($3) }
	|	expr				{ $$=new ExpressionList($1) }
;


expr_without_variable:	
		T_LIST '(' assignment_list ')' '=' expr		{ $$ = new BinaryOperator("=",new AList($3),$6) }
	|	cvar '=' expr					{ $$ = new BinaryOperator("=",$1,$3) }
	|	cvar '=' '&' w_cvar				{ $$ = new BinaryOperator("=",$1,new UnaryOperator("&",$4)) }
	|	cvar '=' '&' function_call			{ $$ = new BinaryOperator("=",$1,new UnaryOperator("&",$4)) }
	|	cvar '=' '&' T_NEW static_or_variable_string    { $$ = new BinaryOperator("=",$1,new UnaryOperator("&",new UnaryOperator("new",$5))) }
                    |                   T_ISSET '(' static_or_variable_string ')' { $$ = new Isset($3) }
	|	T_NEW static_or_variable_string			{ $$ = new UnaryOperator("(new)",$2) }
	|	'(' cvar T_PHPX_VARIABLE expr ')'		{ $$ = new BinaryOperator($3,$2,$4) }
	|	cvar T_PLUS_EQUAL expr				{ $$ = new BinaryOperator("+=",$1,$3) }
	|	cvar T_MINUS_EQUAL expr				{ $$ = new BinaryOperator("-=",$1,$3) }
	|	cvar T_MUL_EQUAL expr				{ $$ = new BinaryOperator("*=",$1,$3) }
	|	cvar T_DIV_EQUAL expr				{ $$ = new BinaryOperator("/=",$1,$3) }
	|	cvar T_CONCAT_EQUAL expr			{ $$ = new BinaryOperator(".=",$1,$3) }
	|	cvar T_MOD_EQUAL expr				{ $$ = new BinaryOperator("%=",$1,$3) }
	|	cvar T_AND_EQUAL expr				{ $$ = new BinaryOperator("&=",$1,$3) }
	|	cvar T_OR_EQUAL expr				{ $$ = new BinaryOperator("|=",$1,$3) }
	|	cvar T_XOR_EQUAL expr				{ $$ = new BinaryOperator("^=",$1,$3) }
	|	cvar T_SL_EQUAL expr				{ $$ = new BinaryOperator("<<=",$1,$3) }
	|	cvar T_SR_EQUAL expr				{ $$ = new BinaryOperator(">>=",$1,$3) }
	|	rw_cvar T_INC					{ $$ = new PostUnaryOperator("++",$1) }
	|	T_INC rw_cvar					{ $$ = new UnaryOperator("++",$2) }
	|	rw_cvar T_DEC					{ $$ = new PostUnaryOperator("--",$1) }
	|	T_DEC rw_cvar					{ $$ = new UnaryOperator("--",$2) }
	|	expr T_BOOLEAN_OR expr				{ $$ = new BinaryOperator("||",$1,$3) }
	|	expr T_BOOLEAN_AND expr				{ $$ = new BinaryOperator("&&",$1,$3) }
	|	expr T_LOGICAL_OR expr				{ $$ = new BinaryOperator("OR",$1,$3) }
	|	expr T_LOGICAL_AND expr				{ $$ = new BinaryOperator("AND",$1,$3) }
	|	expr T_LOGICAL_XOR expr				{ $$ = new BinaryOperator("XOR",$1,$3) }
	|	expr '|' expr					{ $$ = new BinaryOperator("|",$1,$3) }
	|	expr '&' expr					{ $$ = new BinaryOperator("&",$1,$3) }
	|	expr '^' expr					{ $$ = new BinaryOperator("^",$1,$3) }
	|	expr '.' expr					{ $$ = new BinaryOperator(".",$1,$3) }
	|	expr '+' expr					{ $$ = new BinaryOperator("+",$1,$3) }
	|	expr '-' expr					{ $$ = new BinaryOperator("-",$1,$3) }
	|	expr '*' expr					{ $$ = new BinaryOperator("*",$1,$3) }
	|	expr '/' expr					{ $$ = new BinaryOperator("/",$1,$3) }
	|	expr '%' expr					{ $$ = new BinaryOperator("%",$1,$3) }
	| 	expr T_SL expr					{ $$ = new BinaryOperator("<<",$1,$3) }
	|	expr T_SR expr					{ $$ = new BinaryOperator(">>",$1,$3) }
	|	'+' expr					{ $$ = new UnaryOperator("+",$2) }
	|	'-' expr					{ $$ = new UnaryOperator("-",$2) }
	|	'!' expr					{ $$ = new UnaryOperator("!",$2) }
	|	'~' expr					{ $$ = new UnaryOperator("~",$2) }
	|	expr T_IS_IDENTICAL expr			{ $$ = new BinaryOperator("===",$1,$3) }
	|	expr T_IS_NOT_IDENTICAL expr			{ $$ = new BinaryOperator("!==",$1,$3) }
	|	expr T_IS_EQUAL expr				{ $$ = new BinaryOperator("==",$1,$3) }
	|	expr T_IS_NOT_EQUAL expr			{ $$ = new BinaryOperator("!=",$1,$3) }
	|	expr '<' expr 					{ $$ = new BinaryOperator("<",$1,$3) }
	|	expr T_IS_SMALLER_OR_EQUAL expr			{ $$ = new BinaryOperator("<=",$1,$3) }
	|	expr '>' expr 					{ $$ = new BinaryOperator(">",$1,$3) }
	|	expr T_IS_GREATER_OR_EQUAL expr			{ $$ = new BinaryOperator(">=",$1,$3) }
	|	'(' expr ')'					{ $$ = new EncapsExpr("(",$2,")") }
	|	expr '?' expr ':' expr				{ $$ = new AskOperator($1,$3,$5) }
	|	function_call					{ $$ = $1 }
	|	T_INT_CAST expr					{ $$ = new UnaryOperator("(integer)",$2) }
	|	T_DOUBLE_CAST expr				{ $$ = new UnaryOperator("(double)",$2) }
	|	T_STRING_CAST expr				{ $$ = new UnaryOperator("(string)",$2) }
	|	T_ARRAY_CAST expr				{ $$ = new UnaryOperator("(array)",$2) }
	|	T_OBJECT_CAST expr				{ $$ = new UnaryOperator("(object)",$2) }
	|	T_BOOL_CAST expr				{ $$ = new UnaryOperator("(boolean)",$2) }
	|	T_UNSET_CAST expr				{ $$ = new UnaryOperator("(unset)",$2) }
	|	T_EXIT exit_expr				{ $$ = new Exit($2) }
	|	'@' expr					{ $$ = new UnaryOperator("@",$2) }
	|	scalar						{ $$ = $1 }
	|	T_ARRAY '(' array_pair_list ')'			{ $$ = new Array($3) }
	|	'`' encaps_list '`'				{ $$ = new EncapsExpr("`",$2,"`") }
	|	T_PRINT expr					{ $$ = new Print($2) }
	|	T_PHPX_VARIABLE T_VALUE attribute_name     { $$ = new AttributeValue($1,$3,false) }
	|	T_VALUE attribute_name     { $$ = new AttributeValue(new Pin(),$2,false) }
	|	T_PHPX_VARIABLE T_GLOBAL_VALUE attribute_name     { $$ = new AttributeValue($1,$3,true) }
	|	T_GLOBAL_VALUE attribute_name     { $$ = new AttributeValue(new Pin(),$2,true) }
;

function_call:
		T_STRING	'(' function_call_parameter_list ')' { $$ = new FunctionCall(new Pin(),$1,$3) }
	|	cvar '(' function_call_parameter_list ')'	     { $$ = new FunctionCall(new Pin(),$1,$3) }
//	|	T_PHPX_VARIABLE	'(' function_call_parameter_list ')' { $$ = new FunctionCall(new Pin(),$1,$3) }
	|	T_STRING T_PAAMAYIM_NEKUDOTAYIM static_or_variable_string '(' function_call_parameter_list ')' 
							             { $$ = new FunctionCall($1,$3,$5) }
	|	T_PHPX_VARIABLE T_PAAMAYIM_NEKUDOTAYIM T_PHPX_VARIABLE '(' function_call_parameter_list ')' 
							             { $$ = new FunctionCall($1,$3,$5) }
;


static_or_variable_string:
		T_STRING	{ $$ = $1 }
	|	r_cvar		{ $$ = $1 }
;


exit_expr:
		/* empty */	{ $$ = new Pin() }	
	|	'(' ')'		{ $$ = new Pin() }	
	|	'(' expr ')'	{ $$ = $2 }
;

common_scalar:
		T_LNUMBER 					{ $$ = $1 }
	|	T_DNUMBER 					{ $$ = $1 }
	|	T_CONSTANT_ENCAPSED_STRING			{ $$ = new ConstantEncapsedString(new ConstantString($1)) }
	|	T_CONSTANT_ENCAPSED_PHPX_VARIABLE		{ $$ = new ConstantEncapsedString($1) }
	|	T_LINE 						{ $$ = $1 }
	|	T_FILE 						{ $$ = $1 }
	|	T_CLASS_C					{ $$ = $1 }
	|	T_FUNC_C					{ $$ = $1 }
;


static_scalar: /* compile-time evaluated scalars */
		common_scalar		{ $$ = $1 }
	|	T_STRING 		{ $$ = $1 }
	|	'+' static_scalar	{ $$ = new UnaryOperator("+",$2) }
	|	'-' static_scalar	{ $$ = new UnaryOperator("+",$2) }
	|	T_ARRAY '(' static_array_pair_list ')' { $$ = new Array($3) }
;


scalar:
		T_STRING 				{ $$ = $1 }
	|	T_STRING_VARNAME			{ $$ = $1 }
	|	common_scalar				{ $$ = $1 }
	|	'"' encaps_list '"'			{ $$ = new DoubleQuote($2) }
	|	'\'' encaps_list '\''			{ $$ = new EncapsExpr("'",$2,"'") }
	|	T_START_HEREDOC encaps_list T_END_HEREDOC { $$ = new EncapsExpr("<<<",$2,">>>") }
;


static_array_pair_list:
		/* empty */					{ $$ = new ArrayPairList() }
	|	non_empty_static_array_pair_list possible_comma	{ $$ = $1 }
;

possible_comma:
		/* empty */
	|	','
;

non_empty_static_array_pair_list:
		non_empty_static_array_pair_list ',' static_scalar T_DOUBLE_ARROW static_scalar	{ $$ = $1->push_back(new ArrayPair($3,$5)) }
	|	non_empty_static_array_pair_list ',' static_scalar				{ $$ = $1->push_back(new ArrayPair(new Pin(),$3)) }
	|	static_scalar T_DOUBLE_ARROW static_scalar					{ $$ = new ArrayPairList(new ArrayPair($1,$3)) }
	|	static_scalar									{ $$ = new ArrayPairList(new ArrayPair(new Pin(),$1)) }
;

expr:
		r_cvar					{ $$ = $1 }
	|	expr_without_variable			{ $$ = $1 }
;


r_cvar:
	cvar { $$ = $1 }
;


w_cvar:
	cvar { $$ = $1 }
;


rw_cvar:
	cvar { $$ = $1 }
;


cvar:
		cvar_without_objects { $$ = $1 }
	|	cvar_without_objects T_OBJECT_OPERATOR ref_list { $$ = new ObjectOperator($1,$3) }
;


cvar_without_objects:
		reference_variable { $$ = $1 }
	|	'$' cvar_without_objects { $$ = new UnaryOperator("$",$2) }
;


reference_variable:
		reference_variable '[' dim_offset ']'	{ $$ = new ArrayOperator($1,$3) }
	|	reference_variable '{' expr '}'		{ $$ = new MapOperator($1,$3) }
	|	compound_variable			{ $$ = $1 }
;
	

compound_variable:
		T_VARIABLE			{ $$ = new PHPVariable($1) }
	|       phpx_context_variable		{ $$ = $1 }
	|	'$' '{' expr '}'		{ $$ = new MapOperator(new Constant("$"),$3) }
;

dim_offset:
		/* empty */		{ $$ = new Pin() }
	|	expr			{ $$ = $1 }
;

ref_list:
		object_property  { $$ = $1 }
	|	ref_list T_OBJECT_OPERATOR object_property { $$ = new ObjectOperator($1,$3) }
;

object_property:
		object_dim_list { $$ = $1 }
	|	cvar_without_objects { $$ = $1 }
;

object_dim_list:
		object_dim_list '[' dim_offset ']'	{ $$ = new ArrayOperator($1,$3) }
	|	object_dim_list '{' expr '}'		{ $$ = new MapOperator($1,$3) }
	|	variable_name				{ $$ = $1 }
;

variable_name:
		T_STRING		{ $$ = $1 }
	|	'{' expr '}'		{ $$ = new EncapsExpr("{",$2,"}") }
;

assignment_list:
		assignment_list ',' assignment_list_element	{ $$ = $1->push_back($3) }
	|	assignment_list_element				{ $$ = new AssignmentList($1) }
;


assignment_list_element:
		cvar							{ $$ = $1 }
	|	T_LIST '(' assignment_list ')'				{ $$ = new AList($3) }
	|	/* empty */						{ $$ = new Pin() }
;


array_pair_list:
		/* empty */ { $$ = new ArrayPairList() }
	|	non_empty_array_pair_list possible_comma	{ $$=$1 }
;

non_empty_array_pair_list:
		non_empty_array_pair_list ',' expr T_DOUBLE_ARROW expr	{ $$=$1->push_back(new ArrayPair($3,$5)) }
	|	non_empty_array_pair_list ',' expr			{ $$=$1->push_back(new ArrayPair(new Pin(),$3)) }
	|	expr T_DOUBLE_ARROW expr				{ $$=new ArrayPairList(new ArrayPair($1,$3)) }
	|	expr							{ $$=new ArrayPairList(new ArrayPair(new Pin(),$1)) }
	|	non_empty_array_pair_list ',' expr T_DOUBLE_ARROW '&' w_cvar { $$=$1->push_back(new ArrayPair($3,new UnaryOperator("&",$6))) }
	|	non_empty_array_pair_list ',' '&' w_cvar { $$=$1->push_back(new ArrayPair(new Pin(),new UnaryOperator("&",$4))) }
	|	expr T_DOUBLE_ARROW '&' w_cvar	{ $$=new ArrayPairList(new ArrayPair($1,new UnaryOperator("&",$4))) }
	|	'&' w_cvar 			{ $$=new ArrayPairList(new ArrayPair(new Pin(),new UnaryOperator("&",$2))) }
;

encaps_list:
		encaps_list encaps_var			{ $$ = $1->push_back($2) }
	|	encaps_list T_STRING			{ $$ = $1->push_back($2) }
	|	encaps_list T_NUM_STRING		{ $$ = $1->push_back($2) }
	|	encaps_list T_ENCAPSED_AND_WHITESPACE	{ $$ = $1->push_back($2) }
	|	encaps_list T_CHARACTER 		{ $$ = $1->push_back($2) }
	|	encaps_list T_BAD_CHARACTER		{ $$ = $1->push_back($2) }
	|	encaps_list '['				{ $$ = $1->push_back(new Constant("[")) }
	|	encaps_list ']'				{ $$ = $1->push_back(new Constant("]")) }
	|	encaps_list '{'				{ $$ = $1->push_back(new Constant("{")) }
	|	encaps_list '}'				{ $$ = $1->push_back(new Constant("}")) }
	|	encaps_list T_OBJECT_OPERATOR		{ $$ = $1->push_back(new Constant("->")) }
	|	/* empty */				{ $$ = new EncapsList() }

;


encaps_var:
		T_VARIABLE					{ $$ = new PHPVariable($1) }
	|	T_VARIABLE '[' encaps_var_offset ']'		{ $$ = new ArrayOperator(new PHPVariable($1),$3) }
	|	T_VARIABLE T_OBJECT_OPERATOR T_STRING		{ $$ = new ObjectOperator(new PHPVariable($1),$3) }
	|	T_DOLLAR_OPEN_CURLY_BRACES expr '}'		{ $$ = new EncapsExpr("${",$2,"}") }
	|	T_DOLLAR_OPEN_CURLY_BRACES T_STRING_VARNAME '[' expr ']' '}' { $$ = new EncapsExpr("${",
									       new MapOperator($2,$4),"}") }
	|	T_CURLY_OPEN cvar '}'				{ $$=new EncapsExpr("{",$2,"}") }
	|       phpx_context_variable				{ $$ = $1 }
;


encaps_var_offset:
		T_STRING		{ $$=$1 }
	|	T_NUM_STRING		{ $$=$1 }
	|	T_VARIABLE		{ $$=$1 }
;

%%

