#ifndef _PRIORITYLIST_H_
#define _PRIORITYLIST_H_

#include <map>

#ifndef DOXYSKIP
using namespace std;
#endif

/// Template for an item with the priority
template <class Rule>
class PriorityPair : public pair<int,Rule*> {
 public:
  /// Constructor
  PriorityPair(int prioriry, Rule *rule) : pair<int,Rule*>(prioriry,rule) {};
  /// Constructor -- the priority parameter is destroied
  PriorityPair(Constant *prioriry, Rule *rule) : pair<int,Rule*>(prioriry->toInt(),rule) {
    delete prioriry;
  };
};

/// A List of items with priorities
template <class Rule>
class PriorityList : public multimap<int,Rule*>{
 public:

  /// Destructor 
  ~PriorityList(){
    typename multimap<int,Rule*>::iterator rulesIt = begin();
    typename multimap<int,Rule*>::iterator rulesEnd = end();
    while (rulesIt!=rulesEnd){
      delete rulesIt->second;
      rulesIt++;
    };    
  };

  /// Inserts a ::PriorityRulePair in to the list -- inserted pair is destroied
  typename multimap<int,Rule*>::iterator insert(PriorityPair<Rule> *pair) {
    typename multimap<int,Rule*>::iterator it = multimap<int,Rule*>::insert(*pair);
    delete pair;
    return it;
  };

  /// Inserts a ::PriorityRulePair in to the list
  typename multimap<int,Rule*>::iterator insert(const PriorityPair<Rule> &pair) {
    return multimap<int,Rule*>::insert(pair);
  };

  /// Appends list toAppend
  PriorityList* append(const PriorityList &toAppend){
    typename multimap<int,Rule*>::const_iterator it = toAppend.begin();
    typename multimap<int,Rule*>::const_iterator eit = toAppend.end();
    while (it!=eit){
      multimap<int,Rule*>::insert(*it);
      it++;
    };
    return this;
  };
  /// Appedns and frees list toAppend
  PriorityList* append(PriorityList *toAppend){
    append(*toAppend);
    toAppend->clear();
    delete toAppend;
    return this;
  };
};

#endif
