#ifndef _RTREE_H_
#define _RTREE_H_

#include "stree.h"
#include "streeconstant.h"
#include "prioritylist.h"

class RuleContext;

/// Rule for php refactoring.
class Rule{
 public:
  /// Executer the rule on php.
  /*!
    \param context context in which rule is executed 
    \param phpIn input php
    \param phpOut output php
    \param tplOut output smarty template
    \param assign the assignment used to match the rule
    \result true is rule was succefully executed

    %If rule was succefully executed refactored php is deleted from the begining of 
    \a phpIn list. 
   */
    virtual bool execute(RuleContext* context, List* phpIn, List* phpOut, List* tplOut, Node::Assignment& assign) const = 0;

    /// Copies the rule.
    virtual Rule* copy() = 0;
    /// Substitutes variables used in the rule.
    virtual Rule* substitute(const Node::Assignment &assign) = 0;

    /// Writes the rule to the stream.
    virtual ostream& print(ostream& os) const = 0;
    /// Writes the rule to the stream.   
    friend ostream& operator<<(ostream& os, const Rule* rule){
      return rule->print(os);
    };
};

/// Pair containing ::Rule and its priority.
/*!
  Rules with smaller priority are tired at first.
 */
typedef PriorityPair<Rule> PriorityRulePair;

/// List of ::PriorityRulePair
typedef PriorityList<Rule> RuleList;

/// List of rules to run at given time
class RunRuleList : public map<int,RuleList>{
 public:
};

/// A list of rules for a given context
class NamedRuleList{
 public:
  string name;
  int runTime;
  RuleList* rules;

  NamedRuleList(Constant *icontextName, Constant *irunTime,  RuleList *iruleList) : 
    name(icontextName->getValue()), runTime(irunTime->toInt()), rules(iruleList)
    { 
      delete icontextName; 
      delete irunTime;
    };
};


#endif










