#ifndef _RTREEACTIONRULE_H_
#define _RTREEACTIONRULE_H_

#include "rulecontext.h"

/// A virtual class for action that can be performed in rules
class Action{
 public:
  virtual ~Action() {};
  virtual void execute(NameSpace &nameSpace) = 0;
  virtual Action* copy() const = 0;
  virtual Action* substitute(const Node::Assignment &assign) = 0;
};

/// Writes something on the cout
class ActionWrite : public Action{
  /// expresion to be written
  Node* expr;
 public:
  /// constructor
  ActionWrite(Node* iexpr);
  virtual ~ActionWrite();

  virtual void execute(NameSpace &nameSpace);
  virtual Action* copy() const;
  virtual Action* substitute(const Node::Assignment &assign);
};
/// Sets the value of an attribute
class ActionSet : public Action{
  /// A name 
  Node* name;
  /// An attribute
  const string attribute;
  /// Expression that is evalueated
  Node* expr;
  /// Is the name a global name
  const bool global;
 public:
  /// Constructor
  ActionSet(Node* iname, Constant* iattribute, Node* iexpr, bool iglobal);
  virtual ~ActionSet();

  virtual void execute(NameSpace &nameSpace);
  virtual Action* copy() const;
  virtual Action* substitute(const Node::Assignment &assign);
};

/// A list of actions
class ActionList : public list<Action*> {
 public:
  ActionList() {};
  ActionList(Action* action) { push_back(action); };
  ~ActionList();

  ActionList* push_back(Action* action);

  ActionList* copy() const;
  ActionList* substitute(const Node::Assignment &assign);

  virtual void execute(NameSpace &nameSpace) const;
};

/// When the imput is mathced the rule performs some action
class ActionRule : public Rule{
  /// php in pattern
  List* phpInPattern;
  /// action list
  ActionList* actions;
public:
  
  /// Constructor
  ActionRule(List* iphpInPattern, ActionList* iactions);

  virtual ~ActionRule();

  virtual bool execute(RuleContext* context, List* phpIn, List* phpOut, List* tplOut, Node::Assignment& assign) const;
  
  virtual Rule* copy();
  virtual Rule* substitute(const Node::Assignment &assign);
  
  virtual ostream& print(ostream& os) const;
};

/// When the given rule is succesfully executes this rule performs some action
class RuleWithActionRule : public Rule{
  /// php in pattern
  Rule* rule;
  /// action list
  ActionList* actions;
public:
  
  /// Constructor
  RuleWithActionRule(Rule* irule, ActionList* iactions);

  virtual ~RuleWithActionRule();

  virtual bool execute(RuleContext* context, List* phpIn, List* phpOut, List* tplOut, Node::Assignment& assign) const;
  
  virtual Rule* copy();
  virtual Rule* substitute(const Node::Assignment &assign);
  
  virtual ostream& print(ostream& os) const;
};

#endif
