#include "rtreecontextrule.h"
#include "streevariable.h"

ContextRule::ContextRule(List* iphpInPattern, List* itplOutPattern, List* iphpOutPattern) :
    phpInPattern(iphpInPattern), tplOutPattern(itplOutPattern),phpOutPattern(iphpOutPattern) {}

ContextRule::~ContextRule(){
  delete phpInPattern;
  delete phpOutPattern;
  delete tplOutPattern;
};

Rule* ContextRule::substitute(const Node::Assignment &assign){
  phpInPattern->substitute(assign);
  tplOutPattern->substitute(assign);
  phpOutPattern->substitute(assign);
  return this;
};


bool ContextRule::execute(RuleContext* context, List* phpIn, List* phpOut, List* tplOut, Node::Assignment& assign)const {
    if (phpInPattern->matchToBegining(phpIn,assign, context->getNameSpace())){
      RuleContext* subcontext = generateSubcontext(context,assign);

      StatementList *tplOutMatched = new StatementList();

      bool tplOutIsEmpty = true;

      List::iterator it = tplOutPattern->begin();
      List::iterator eit = tplOutPattern->end();
      while (it!=eit){
	if ((*it)->getName()==CVariable){
	  Node* phpContextMatched = (*it)->copy();
	  substituteNode(phpContextMatched,assign);
	  
	  StatementList* phpContextOut = new StatementList();
	  
	  unsigned int tplOutMatchedLenght = tplOutMatched->size();
	  subcontext->execute(phpContextMatched,phpContextOut,tplOutMatched);
	  if (tplOutMatchedLenght != tplOutMatched->size())
	    tplOutIsEmpty = false;

	  assign[dynamic_cast<Variable*>(*it)->getId()] = phpContextOut;

	}else if ((*it)->getName()==CTemplate){
	  Node* tplMatched = (*it)->copy();
	  substituteNode(tplMatched,assign);
	  subcontext->processTemplate(tplMatched);
	  tplOutMatched->push_back(tplMatched);
	}else throw "ERROR: ContextRule::execute - Wrong element in template list";
	it++;
      }
	
      StatementList *phpOutMatched = dynamic_cast<StatementList*>(phpOutPattern->copy());
      phpOutMatched->substitute(assign);
      
      for (int i=0 ; i<phpInPattern->getLength() ; i++){
	delete phpIn->front();
	phpIn->pop_front();
      };

      subcontext->assignVariableNames(phpOutMatched,tplOutMatched);

      delete subcontext;
       
      // HMMM: moze nie potrzebne?
      context->assignVariableNames(phpOutMatched,tplOutMatched);

      phpOut->append(phpOutMatched);
      if (!tplOutIsEmpty)
	tplOut->append(tplOutMatched);
      return true;
    }
    return false;
};

