#include "rtreeincluderule.h"
#include "singlecontext.h"
#include "config.h"

IncludeRule::IncludeRule(Constant *iname, RunRuleList &irunRuleList) :
    name(iname->getValue()),
    runRuleList(irunRuleList) {
  delete iname;
};

IncludeRule::~IncludeRule(){
};

bool IncludeRule::execute(RuleContext* context, List* phpIn, List* phpOut, List* tplOut, Node::Assignment& assign) const {
  RunRuleList::iterator ruleListIt = runRuleList.begin();
  RunRuleList::iterator ruleListEnd = runRuleList.end();
  if (verbose>2) cout << "Doing: " << (phpIn->front()) << endl;
  if (verbose>2) phpIn->front()->print(cout) << endl;

  if (ruleListIt==ruleListEnd)
    throw "ERROR: IncludeRule::runRules - no rules provided for context " + name; 
  while (ruleListIt != ruleListEnd){
    RuleList::const_iterator rulesIt = ruleListIt->second.begin();
    RuleList::const_iterator rulesEnd = ruleListIt->second.end();
    while (rulesIt!=rulesEnd){
      if (verbose>2) cout << "Trying:" << rulesIt->second << endl;
      if (rulesIt->second->execute(context,phpIn,phpOut,tplOut,assign)) {
      	if (verbose>2) cout << " Success" << endl;
      	return true;
      }      
      if (verbose>2) cout << " Failed" << endl;
      rulesIt++;
    };
    ruleListIt++;
  };
  return false;
};

ostream& IncludeRule::print(ostream& os) const{
  os << "IncludeRule - " << name;
  return os;
};

Rule* IncludeRule::copy(){
  throw "ERROR: IncludeRule::copy called";
};

Rule* IncludeRule::substitute(const Node::Assignment &assign){
  throw "ERROR: IncludeRule::substitute called";
};
