#include "rtreenamedcontextrule.h"
#include "namedcontext.h"

NamedContextRule::NamedContextRule(Constant *iname, RunRuleList &irunRuleList, List* iphpInPattern, List* iphpOutPattern, 
				   Variable* inamePattern, Variable* iargumentsPattern, 
				   Variable* icontextPattern) :
    name(iname->getValue()),
    runRuleList(irunRuleList), 
    phpInPattern(iphpInPattern), 
    phpOutPattern(iphpOutPattern), 
    namePattern(inamePattern), 
    argumentsPattern(iargumentsPattern), 
    contextPattern(icontextPattern) {
  delete iname;
};

NamedContextRule::~NamedContextRule(){
  delete phpInPattern;
  delete namePattern;
  delete argumentsPattern;
  delete contextPattern;
};

void NamedContextRule::runRules(NameSpace &subNameSpace, Node* phpContextMatched, StatementList* phpContextOut, 
				StatementList* tplContextOut, Node::Assignment& assign) const {

  RunRuleList::iterator ruleListIt = runRuleList.find(subNameSpace.getRuleRunTime());
  if (ruleListIt!=runRuleList.end()){
    NamedContext subcontext(name,ruleListIt->second,subNameSpace);
    subcontext.execute(phpContextMatched, phpContextOut, tplContextOut);
    if ((tplContextOut->getLength()) && (name != MAINCONTEXTNAME))
      throw "NamedContextRule::execute - template output is not empty";
    ruleListIt++;
  };
}


bool NamedContextRule::execute(RuleContext* context, List* phpIn, List* phpOut, List* tplOut, Node::Assignment& assign) const {
  if (phpInPattern->matchToBegining(phpIn,assign, context->getNameSpace())){

    Node* phpNameMatched = namePattern->copy();
    substituteNode(phpNameMatched, assign);
    Node* phpArgumentsMatched = argumentsPattern->copy();
    substituteNode(phpArgumentsMatched,assign);
    Node* phpContextMatched = contextPattern->copy();
    substituteNode(phpContextMatched,assign);

    NameSpace subNameSpace(context->getNameSpace());
    subNameSpace.processArguments(phpArgumentsMatched);

    StatementList *phpContextOut = new StatementList();
    StatementList *tplContextOut = new StatementList();

    runRules(subNameSpace, phpContextMatched, phpContextOut, tplContextOut, assign);
    
    assign[contextPattern->getId()] = phpContextOut;
    StatementList *phpOutMatched = dynamic_cast<StatementList*>(phpOutPattern->copy());
    phpOutMatched->substitute(assign);
    phpOut->append(phpOutMatched);
    tplOut->append(tplContextOut);

    for (int i=0 ; i<phpInPattern->getLength() ; i++){
      delete phpIn->front();
      phpIn->pop_front();
    };

    context->getNameSpace().importLocals(phpNameMatched->toString(), subNameSpace);
    return true;
  }
  return false;
};

ostream& NamedContextRule::print(ostream& os) const{
      os << "NamedContextRule - ";
      phpInPattern->print(os);
      //      tplOutPattern->print(os);
      return os;
};

Rule* NamedContextRule::copy(){
  throw "ERROR: NamedContextRule::copy called";
};

Rule* NamedContextRule::substitute(const Node::Assignment &assign){
  throw "ERROR: NamedContextRule::substitute called";
};
