#ifndef _RTREENAMEDCONTEXTRULE_H_
#define _RTREENAMEDCONTEXTRULE_H_

#include "rtreecontextrule.h"
#include "streevariable.h"

/// Used for executing rules from other files on a statement list
class NamedContextRule : public Rule{
  
 private:
  /// Name of rules to include
  const string name;
  /// Reference to rule list in ::RuleSystem
  RunRuleList &runRuleList;

  /// Pattern to be matched
  List* phpInPattern;
  /// Output pattern
  List* phpOutPattern;
  /// The name of the context
  Variable* namePattern;
  /// The list of arguments
  Variable* argumentsPattern;
  /// The statements of the context
  Variable* contextPattern;

 public:
  /// Constructor
  NamedContextRule(Constant* iname, RunRuleList &irunRuleList, List* iphpInPattern, List* iphpOutPattern, 
		   Variable* inamePattern,   Variable* iargumentsPattern, Variable* icontextPattern); 

  virtual ~NamedContextRule();
  
  virtual bool execute(RuleContext* context, List* phpIn, List* phpOut, List* tplOut, Node::Assignment& assign) const;

  /// Executes rules on given statement list used in ::RuleSystem for main statement list.
  void runRules(NameSpace &subNameSpace, Node* phpContextMatched, StatementList* phpContextOut, 
		StatementList* tplContextOut, Node::Assignment& assign) const;

  virtual Rule* copy();

  virtual Rule* substitute(const Node::Assignment &assign);
  
  virtual ostream& print(ostream& os) const;
};


#endif
