#include "rulecontext.h"
#include "config.h"
#include <sstream>

bool RuleContext::executeOnPHP(List* phpIn, List* phpOut, List* tplOut){
    RuleList::const_iterator rulesIt = rules.begin();
    RuleList::const_iterator rulesEnd = rules.end();
    if (verbose>2) cout << "Doing: " << (phpIn->front()) << endl;
    if (verbose>2) phpIn->front()->print(cout) << endl;
    while (rulesIt!=rulesEnd){
      if (verbose>2) cout << "Trying:" << rulesIt->second << endl;
      Node::Assignment assign;
      if (rulesIt->second->execute(this,phpIn,phpOut,tplOut,assign)) {
      	if (verbose>2) cout << " Success" << endl;
      	return true;
      }      
      if (verbose>2) cout << " Failed" << endl;
      rulesIt++;
    };
    return false;
    /*
    if (phpIn->front()->getName()!=CInlineHTMLList)
      phpOut->push_back(phpIn->front());
    else 
      tplOut->push_back(phpIn->front());
    phpIn->pop_front();*/
};

void RuleContext::executeOnPHPList(List* phpIn, List *phpOut, List *tplOut){
    while (phpIn->getLength()){
      if (!executeOnPHP(phpIn,phpOut, tplOut))
	throw "ERROR: RuleContext::executeOnPHPList - no rule provided for : " + phpIn->front()->toString();
    }
    delete phpIn;
};

RuleContext::RuleContext(RuleList &irules, NameSpace &inameSpace) : rules(irules), nameSpace(inameSpace) {};

RuleContext::~RuleContext(){
    list<RuleList::iterator>::iterator it = myRules.begin();
    list<RuleList::iterator>::iterator eit = myRules.end();
    while(it!=eit){
      delete (*it)->second;
      rules.erase(*it);
      it++;
    };
};


void RuleContext::insert(int priority, Rule *rule){
  myRules.push_back(rules.insert(PriorityRulePair(priority,rule)));
};

void RuleContext::insert(PriorityRulePair *rulePair){
  myRules.push_back(rules.insert(rulePair));
};

void RuleContext::execute(Node* phpIn,List *phpOut, List *tplOut){
    enterContext(phpOut,tplOut);
    if (verbose>2) cout << "IN EXECUTE" << endl;
    if (phpIn->getName() == CStatementList)
      executeOnPHPList(dynamic_cast<StatementList*>(phpIn),phpOut,tplOut);
    else
      executeOnPHPList(new StatementList(phpIn),phpOut,tplOut);
    if (verbose>2) cout << "OUT EXECUTE" << endl;

    leaveContext(phpOut,tplOut);
};

void RuleContext::assignVariableNames(List *phpList, List *tplList){
      Node::VariableList vlist;
      tplList->getVariables(vlist);
      phpList->getVariables(vlist);
      Node::Assignment phpAssign;      
      Node::Assignment tplAssign;      

      generateVariableNames(vlist,phpAssign,tplAssign);

      phpList->substitute(phpAssign);
      tplList->substitute(tplAssign);

      processSmartyAssignInPhpList(phpList);
};
    
void RuleContext::processSmartyAssignInPhpList(List* phpList){
    List::iterator it = phpList->begin();
    List::iterator eit = phpList->end();

    while (it!=eit){
      if ((*it)->getName() == CSmartyAssign)
	processSmartyAssign(dynamic_cast<SmartyAssign*>(*it));
      it++;
    };
    
};

void RuleContext::enterContext(List *phpOut, List *tplOut) {};
void RuleContext::leaveContext(List *phpOut, List *tplOut) {};


