#ifndef _RULESYSTEM_H_
#define _RULESYSTEM_H_

#define MYRULES "rules/"


#include "stree.h"
#include "rtree.h"
#include "namespace.h"

/// Refactoring system
class RuleSystem {
 public:
  /// A run rule list for every context name 
  typedef map<string, RunRuleList> NamedRunRuleList;

 private:

  /// Rules of the system
  NamedRunRuleList runRules;
  /// The main name space -- for globals
  NameSpace mainNameSpace;
  /// All run times -- used internally
  set<int> ruleRunTimes;
  /// Courent run time
  set<int>::iterator courrentRunTime;
 public:

  /// Constructor
  RuleSystem(int debug = 0);

  /// Executes system on a given program
  void execute(const string &phpFileName, StatementList *phpIn, StatementList *phpOut, StatementList *tplOut);

  /// Destructor
  ~RuleSystem();

  /// Return rule list for a given context
  RunRuleList& getRunRuleList(const string &contextName) { return runRules[contextName]; }; 

  /// Steps to the next run time
  bool stepTime();
  /// Returns the cournent run time
  int getRuleRunTime() const { return *courrentRunTime; };
};

#endif
