
%{
#include "stree.h"
#include "streevariable.h"
#include "rtree.h"
#include "testers.h"
#include "rtreeactionrule.h"
#include "parser.tab.h"
#include "FlexLexer.h"
#include "treeparser.h"
#include <errno.h>
#include <string.h>
#include <iostream>
%}



%x ST_IN_SCRIPTING
%x ST_DOUBLE_QUOTES
%x ST_SINGLE_QUOTE
%x ST_BACKQUOTE
%x ST_HEREDOC
%x ST_LOOKING_FOR_PROPERTY
%x ST_LOOKING_FOR_VARNAME
%x ST_COMMENT
%x ST_ONE_LINE_COMMENT
%option stack
%option noyywrap
%{

int parseRules = false;

int heredoc_len = 0;
char *heredoc = NULL;
int yylineno = 1;

#define YY_DECL int yylexx(void)

static int yy_top_state() __attribute__ ((unused));

//#define ECHO { ZEND_WRITE( yytext, yyleng ); }

//#define YY_FATAL_ERROR zend_fatal_scanner_error

#define asp_tags true
#define short_tags true

#define HANDLE_NEWLINES(s, l)				\
do {	   						\
	char *p = (s), *boundary = p+(l);		\
	 						\
	while (p<boundary) {				\
		if (*p == '\n') {			\
			++(yylineno);			\
		} else if ((*p == '\r') && (p+1 < boundary) && (*(p+1) != '\n')) { \
			++(yylineno);			\
		}					\
		p++;					\
	}						\
} while (0)

#define HANDLE_NEWLINE(c) \
{ \
	if (c == '\n' || c == '\r') { \
		(yylineno)++; \
	} \
}
		

#define ZEND_IS_OCT(c)  ((c)>='0' && (c)<='7')
#define ZEND_IS_HEX(c)  (((c)>='0' && (c)<='9') || ((c)>='a' && (c)<='f') || ((c)>='A' && (c)<='F'))

bool wasSemicolon = false;
bool sendCloseTag = false;

int yylexx(void);

int yylex(void){
    int result;
    if (sendCloseTag) {
       sendCloseTag = false;
       return  T_CLOSE_TAG;
    }
    result = yylexx();
    switch (result){
    case ';':
	 wasSemicolon = true;
	 break;
    case T_COMMENT:
	 break;
    case T_CLOSE_TAG:
	 if (!wasSemicolon){
	    sendCloseTag = true;
	    result = ';';
	 }   
    default:
	 wasSemicolon = false;
    };
    return result;
};


void error(char *message,char *pos)
{
	cout << message << " " << pos << endl;
        exit(1);
}

void yyerror(const char *message)
{
	cout << "line " << yylineno << ": " <<message << endl;
        exit(1);
}

void warning(char *message,char *pos)
{
	cout << message << " " << pos << endl;
        exit(1);
}

char* strndup(const char *s, uint length)
{
	char *p;

	p = (char *) malloc(length+1);
	if (!p) {
		return (char *)NULL;
	}
	if (length) {
		memcpy(p, s, length);
	}
	p[length] = 0;
	return p;
}

%}

LNUM	[0-9]+
DNUM	([0-9]*[\.][0-9]+)|([0-9]+[\.][0-9]*)
EXPONENT_DNUM	(({LNUM}|{DNUM})[eE][+-]?{LNUM})
HNUM	"0x"[0-9a-fA-F]+
LABEL	[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*
ENCAPSED_STRING ([a-zA-Z0-9_\x7f-\xff \t #'.:;,()|^&+/*=%!~<>?@]|"-"[a-zA-Z0-9_\x7f-\xff \t #'.:;,()|^&+/*=%!~<?@])+
ENCAPSED_STRING_WITH_NEWLINE ([a-zA-Z0-9_\x7f-\xff \t\n\r #'.:;,()|^&+/*=%!~<>?@]|"-"[a-zA-Z0-9_\x7f-\xff \t\n\r #'.:;,()|^&+/*=%!~<?@])+
WHITESPACE [ \n\r\t]+
TABS_AND_SPACES [ \t]*
TOKENS [;:,.\[\]()|^&+-/*=%!~$<>?@]
ENCAPSED_TOKENS [\[\]{}$]
ESCAPED_AND_WHITESPACE [\n\t\r #'.:;,()|^&+-/*=%!~<>?@]+
ANY_CHAR (.|[\n])
NEWLINE ("\r"|"\n"|"\r\n")

%%

<ST_IN_SCRIPTING>"substitutionrule" {
        if (parseRules == ParseRules)
	   return T_SUBSTITUTION_RULE;
	else 
	   REJECT;	
}

<ST_IN_SCRIPTING>"singlecontextrule" {
        if (parseRules == ParseRules)
	   return T_SINGLE_CONTEXT_RULE;
	else 
	   REJECT;	
}

<ST_IN_SCRIPTING>"multicontextrule" {
        if (parseRules == ParseRules)
	   return T_MULTI_CONTEXT_RULE;
	else 
	   REJECT;	
}

<ST_IN_SCRIPTING>"namedcontextrule" {
        if (parseRules == ParseRules)
	   return T_NAMED_CONTEXT_RULE;
	else 
	   REJECT;	
}

<ST_IN_SCRIPTING>"triggerrule" {
        if (parseRules == ParseRules)
	   return T_TRIGGERED_RULE;
	else 
	   REJECT;	
}
<ST_IN_SCRIPTING>"includerule" {
        if (parseRules == ParseRules)
	   return T_INCLUDE_RULE;
	else 
	   REJECT;	
}

<ST_IN_SCRIPTING>"normalize" {
        if (parseRules == ParseRules)
	   return T_NORMALIZE;
	else 
	   REJECT;
}

<ST_IN_SCRIPTING>"template" {
        if (parseRules == ParseRules)
	   return T_TEMPLATE;
	else 
	   REJECT;
}

<ST_IN_SCRIPTING>"smartyassign" {
        if (parseRules == ParseRules)
	   return T_SMARTY_ASSIGN;
	else 
	   REJECT;
}

<ST_IN_SCRIPTING>"name" {
        if (parseRules == ParseRules)
	   return T_NAME;
	else 
	   REJECT;
}

<ST_IN_SCRIPTING>"type" {
        if (parseRules == ParseRules)
	   return T_TYPE;
	else 
	   REJECT;
}

<ST_IN_SCRIPTING>"exists" {
        if (parseRules == ParseRules)
	   return T_EXISTS;
	else 
	   REJECT;
}
<ST_IN_SCRIPTING>"forall" {
        if (parseRules == ParseRules)
	   return T_FORALL;
	else 
	   REJECT;
}
<ST_IN_SCRIPTING>"not" {
        if (parseRules == ParseRules)
	   return T_NOT;
	else 
	   REJECT;
}

<ST_IN_SCRIPTING>"isequal" {
        if (parseRules == ParseRules)
	   return T_ISEQUAL;
	else 
	   REJECT;
}
<ST_IN_SCRIPTING>"tisset" {
        if (parseRules == ParseRules)
	   return T_TISSET;
	else 
	   REJECT;
}


<ST_IN_SCRIPTING>"value" {
        if (parseRules == ParseRules)
	   return T_VALUE;
	else 
	   REJECT;
}
<ST_IN_SCRIPTING>"globalvalue" {
        if (parseRules == ParseRules)
	   return T_GLOBAL_VALUE;
	else 
	   REJECT;
}

<ST_IN_SCRIPTING>"foralltesters"|"tand" {
        if (parseRules == ParseRules)
	   return T_FORALL_TESTERS;
	else 
	   REJECT;
}
<ST_IN_SCRIPTING>"existstester"|"tor" {
        if (parseRules == ParseRules)
	   return T_EXISTS_TESTER;
	else 
	   REJECT;
}

<ST_IN_SCRIPTING>"actionrule" {
        if (parseRules == ParseRules)
	   return T_ACTION_RULE;
	else 
	   REJECT;
}
<ST_IN_SCRIPTING>"write" {
        if (parseRules == ParseRules)
	   return T_WRITE;
	else 
	   REJECT;
}
<ST_IN_SCRIPTING>"set" {
        if (parseRules == ParseRules)
	   return T_SET;
	else 
	   REJECT;
}

<ST_IN_SCRIPTING>"die" {
	return T_EXIT;
}

<ST_IN_SCRIPTING>"old_function" {
	return T_OLD_FUNCTION;
}

<ST_IN_SCRIPTING>"function"|"cfunction" {
	return T_FUNCTION;
}

<ST_IN_SCRIPTING>"const" {
	return T_CONST;
}

<ST_IN_SCRIPTING>"return" {
	return T_RETURN;
}

<ST_IN_SCRIPTING>"if" {
	return T_IF;
}

<ST_IN_SCRIPTING>"elseif" {
	return T_ELSEIF;
}

<ST_IN_SCRIPTING>"endif" {
	return T_ENDIF;
}

<ST_IN_SCRIPTING>"else" {
	return T_ELSE;
}

<ST_IN_SCRIPTING>"while" {
	return T_WHILE;
}

<ST_IN_SCRIPTING>"endwhile" {
	return T_ENDWHILE;
}

<ST_IN_SCRIPTING>"do" {
	return T_DO;
}

<ST_IN_SCRIPTING>"for" {
	return T_FOR;
}

<ST_IN_SCRIPTING>"endfor" {
	return T_ENDFOR;
}

<ST_IN_SCRIPTING>"foreach" {
	return T_FOREACH;
}

<ST_IN_SCRIPTING>"endforeach" {
	return T_ENDFOREACH;
}

<ST_IN_SCRIPTING>"declare" {
	return T_DECLARE;
}

<ST_IN_SCRIPTING>"enddeclare" {
	return T_ENDDECLARE;
}

<ST_IN_SCRIPTING>"as" {
	return T_AS;
}

<ST_IN_SCRIPTING>"switch" {
	return T_SWITCH;
}

<ST_IN_SCRIPTING>"endswitch" {
	return T_ENDSWITCH;
}

<ST_IN_SCRIPTING>"case" {
	return T_CASE;
}

<ST_IN_SCRIPTING>"default" {
	return T_DEFAULT;
}

<ST_IN_SCRIPTING>"break" {
	return T_BREAK;
}

<ST_IN_SCRIPTING>"continue" {
	return T_CONTINUE;
}

<ST_IN_SCRIPTING>"echo" {
	return T_ECHO;
}


<ST_IN_SCRIPTING>"print" {
	return T_PRINT;
}

<ST_IN_SCRIPTING>"class" {
	return T_CLASS;
}

<ST_IN_SCRIPTING>"extends" {
	return T_EXTENDS;
}

<ST_IN_SCRIPTING,ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>"->" {
	yy_push_state(ST_LOOKING_FOR_PROPERTY);
	return T_OBJECT_OPERATOR;
}

<ST_LOOKING_FOR_PROPERTY>{LABEL} {
	yy_pop_state();
// 	zend_copy_value(zendlval, yytext, yyleng);
//	zendlval->value.str.len = yyleng;
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_STRING;
}

<ST_LOOKING_FOR_PROPERTY>{ANY_CHAR} {
	yyless(0);
	yy_pop_state();
}

<ST_IN_SCRIPTING>"::" {
	return T_PAAMAYIM_NEKUDOTAYIM;
}

<ST_IN_SCRIPTING>"new" {
	return T_NEW;
}

<ST_IN_SCRIPTING>"var" {
	return T_VAR;
}

<ST_IN_SCRIPTING>"("{TABS_AND_SPACES}("int"|"integer"){TABS_AND_SPACES}")" {
	return T_INT_CAST;
}

<ST_IN_SCRIPTING>"("{TABS_AND_SPACES}("real"|"double"|"float"){TABS_AND_SPACES}")" {
	return T_DOUBLE_CAST;
}

<ST_IN_SCRIPTING>"("{TABS_AND_SPACES}"string"{TABS_AND_SPACES}")" {
	return T_STRING_CAST;
}

<ST_IN_SCRIPTING>"("{TABS_AND_SPACES}"array"{TABS_AND_SPACES}")" {
	return T_ARRAY_CAST;
}

<ST_IN_SCRIPTING>"("{TABS_AND_SPACES}"object"{TABS_AND_SPACES}")" {
	return T_OBJECT_CAST;
}

<ST_IN_SCRIPTING>"("{TABS_AND_SPACES}("bool"|"boolean"){TABS_AND_SPACES}")" {
	return T_BOOL_CAST;
}

<ST_IN_SCRIPTING>"("{TABS_AND_SPACES}("unset"){TABS_AND_SPACES}")" {
	return T_UNSET_CAST;
}

<ST_IN_SCRIPTING>"eval" {
	return T_EVAL;
}

<ST_IN_SCRIPTING>"include" {
	return T_INCLUDE;
}

<ST_IN_SCRIPTING>"include_once" {
	return T_INCLUDE_ONCE;
}

<ST_IN_SCRIPTING>"require" {
	return T_REQUIRE;
}

<ST_IN_SCRIPTING>"require_once" {
	return T_REQUIRE_ONCE;
}

<ST_IN_SCRIPTING>"use" {
	return T_USE;
}

<ST_IN_SCRIPTING>"global" {
	return T_GLOBAL;
}

<ST_IN_SCRIPTING>"isset" {
	return T_ISSET;
}

<ST_IN_SCRIPTING>"empty" {
	return T_EMPTY;
}

<ST_IN_SCRIPTING>"static" {
	return T_STATIC;
}

<ST_IN_SCRIPTING>"unset" {
	return T_UNSET;
}

<ST_IN_SCRIPTING>"=>" {
	return T_DOUBLE_ARROW;
}

<ST_IN_SCRIPTING>"list" {
	return T_LIST;
}

<ST_IN_SCRIPTING>"array" {
	return T_ARRAY;
}

<ST_IN_SCRIPTING>"++" {
	return T_INC;
}

<ST_IN_SCRIPTING>"--" {
	return T_DEC;
}

<ST_IN_SCRIPTING>"===" {
	return T_IS_IDENTICAL;
}

<ST_IN_SCRIPTING>"!==" {
	return T_IS_NOT_IDENTICAL;
}

<ST_IN_SCRIPTING>"==" {
	return T_IS_EQUAL;
}

<ST_IN_SCRIPTING>"!="|"<>" {
	return T_IS_NOT_EQUAL;
}

<ST_IN_SCRIPTING>"<=" {
	return T_IS_SMALLER_OR_EQUAL;
}

<ST_IN_SCRIPTING>">=" {
	return T_IS_GREATER_OR_EQUAL;
}

<ST_IN_SCRIPTING>"+=" {
	return T_PLUS_EQUAL;
}

<ST_IN_SCRIPTING>"-=" {
	return T_MINUS_EQUAL;
}

<ST_IN_SCRIPTING>"*=" {
	return T_MUL_EQUAL;
}

<ST_IN_SCRIPTING>"/=" {
	return T_DIV_EQUAL;
}

<ST_IN_SCRIPTING>".=" {
	return T_CONCAT_EQUAL;
}

<ST_IN_SCRIPTING>"%=" {
	return T_MOD_EQUAL;
}

<ST_IN_SCRIPTING>"<<=" {
	return T_SL_EQUAL;
}

<ST_IN_SCRIPTING>">>=" {
	return T_SR_EQUAL;
}

<ST_IN_SCRIPTING>"&=" {
	return T_AND_EQUAL;
}

<ST_IN_SCRIPTING>"|=" {
	return T_OR_EQUAL;
}

<ST_IN_SCRIPTING>"^=" {
	return T_XOR_EQUAL;
}

<ST_IN_SCRIPTING>"||" {
	return T_BOOLEAN_OR;
}

<ST_IN_SCRIPTING>"&&" {
	return T_BOOLEAN_AND;
}

<ST_IN_SCRIPTING>"OR" {
	return T_LOGICAL_OR;
}

<ST_IN_SCRIPTING>"AND" {
	return T_LOGICAL_AND;
}

<ST_IN_SCRIPTING>"XOR" {
	return T_LOGICAL_XOR;
}

<ST_IN_SCRIPTING>"<<" {
	return T_SL;
}

<ST_IN_SCRIPTING>">>" {
	return T_SR;
}

<ST_IN_SCRIPTING>{TOKENS} {
	return yytext[0];
}


<ST_IN_SCRIPTING>"{" {
	yy_push_state(ST_IN_SCRIPTING);
	return '{';
}


<ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>"${" {
	yy_push_state(ST_LOOKING_FOR_VARNAME);
	return T_DOLLAR_OPEN_CURLY_BRACES;
}


<ST_IN_SCRIPTING>"}" {
	/* This is a temporary fix which is dependant on flex and it's implementation */
	if (yy_start_stack_ptr) {
		yy_pop_state();
	}
	return '}';
}


<ST_LOOKING_FOR_VARNAME>{LABEL} {
// 	zend_copy_value(zendlval, yytext, yyleng);
//	zendlval->type = IS_STRING;
	yy_pop_state();
	yy_push_state(ST_IN_SCRIPTING);
	yylval.constant_val = new Constant(yytext);
	return T_STRING_VARNAME;
}


<ST_LOOKING_FOR_VARNAME>{ANY_CHAR} {
	yyless(0);
	yy_pop_state();
	yy_push_state(ST_IN_SCRIPTING);
}


<ST_IN_SCRIPTING>{LNUM} {
	errno = 0;
//	zendlval->value.lval = strtol(yytext, NULL, 0);
	if (errno == ERANGE) { /* overflow */
//		zendlval->value.dval = strtod(yytext, NULL);
//		zendlval->type = IS_DOUBLE;
		yylval.constant_val = new Constant(yytext);
		return T_DNUMBER;
	} else {
//		zendlval->type = IS_LONG;
		yylval.constant_val = new Constant(yytext);
		return T_LNUMBER;
	}
}

<ST_IN_SCRIPTING>{HNUM} {
	errno = 0;
//	zendlval->value.lval = strtoul(yytext, NULL, 16);
	if (errno == ERANGE) { /* overflow */
		/* not trying strtod - it returns trash on 0x-es */
//		zendlval->value.lval = LONG_MAX; /* maximal long */
		error("Hex number is too big: %s", yytext);
	} else {
//		if (zendlval->value.lval < 0) {
			/* maintain consistency with the old way */
//			zendlval->value.dval = (unsigned long) zendlval->value.lval;
//			zendlval->type = IS_DOUBLE;
			yylval.constant_val = new Constant(yytext);
			return T_DNUMBER;
//		}
//		zendlval->type = IS_LONG;
	}
//	zendlval->type = IS_LONG;
	yylval.constant_val = new Constant(yytext);
	return T_LNUMBER;
}

<ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>{LNUM}|{HNUM} { /* treat numbers (almost) as strings inside encapsulated strings */
//	zendlval->value.str.val = (char *)estrndup(yytext, yyleng);
//	zendlval->value.str.len = yyleng;
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_NUM_STRING;
}

<ST_IN_SCRIPTING>{DNUM}|{EXPONENT_DNUM} {
//	zendlval->value.dval = strtod(yytext, NULL);
//	zendlval->type = IS_DOUBLE;
	yylval.constant_val = new Constant(yytext);
	return T_DNUMBER;
}

<ST_IN_SCRIPTING>"__CLASS__" {
/*	char *class_name = NULL;

	if (CG(active_class_entry)) {
		class_name = CG(active_class_entry)->name;
	}
	
	if (!class_name) {
		class_name = "";
	}
	zendlval->value.str.len = strlen(class_name);
	zendlval->value.str.val = estrndup(class_name, zendlval->value.str.len);
	zendlval->type = IS_STRING;*/
	error("__CLASS__",yytext);
	yylval.constant_val = new Constant(yytext);
	return T_CLASS_C;
}

<ST_IN_SCRIPTING>"__FUNCTION__" {
/*	char *func_name = CG(active_op_array)->function_name;
	
	if (!func_name) {
		func_name = "";
	}
	zendlval->value.str.len = strlen(func_name);
	zendlval->value.str.val = estrndup(func_name, zendlval->value.str.len);
	zendlval->type = IS_STRING;*/
	yylval.constant_val = new Constant(yytext);
	error("__FUNCTION__",yytext);
	return T_FUNC_C;
}

<ST_IN_SCRIPTING>"__LINE__" {
//	zendlval->value.lval = CG(zend_lineno);
//	zendlval->type = IS_LONG;
	error("__LINE__",yytext);
	yylval.constant_val = new Constant(yytext);
	return T_LINE;
}

<ST_IN_SCRIPTING>"__FILE__" {
/*	char *filename = zend_get_compiled_filename(TSRMLS_C);
	
	if (!filename) {
		filename = "";
	}
	zendlval->value.str.len = strlen(filename);
	zendlval->value.str.val = estrndup(filename, zendlval->value.str.len);
	zendlval->type = IS_STRING;*/
	error("__FILE__",yytext);
	yylval.constant_val = new Constant(yytext);
	return T_FILE;
}


<INITIAL>(([^<]|"<"[^?%s<]){1,400})|"<s"|"<" {
//	zendlval->value.str.val = (char *) estrndup(yytext, yyleng);
//	zendlval->value.str.len = yyleng;
//	zendlval->type = IS_STRING;
	HANDLE_NEWLINES(yytext, yyleng);
	yylval.constant_val = new Constant(yytext);
	return T_INLINE_HTML;
}

<INITIAL>"<?"|"<script"{WHITESPACE}+"language"{WHITESPACE}*"="{WHITESPACE}*("php"|"\"php\""|"\'php\'"){WHITESPACE}*">" {
	HANDLE_NEWLINES(yytext, yyleng);
	if (short_tags || yyleng>2) { /* yyleng>2 means it's not <? but <script> */
//		zendlval->value.str.val = yytext; /* no copying - intentional */
//		zendlval->value.str.len = yyleng;
//		zendlval->type = IS_STRING;
		BEGIN(ST_IN_SCRIPTING);
		return T_OPEN_TAG;
	} else {
//		zendlval->value.str.val = (char *) estrndup(yytext, yyleng);
//		zendlval->value.str.len = yyleng;
//		zendlval->type = IS_STRING;
		yylval.constant_val = new Constant(yytext);	
		return T_INLINE_HTML;
	}
}


<INITIAL>"<%="|"<?=" {
	if ((yytext[1]=='%' && asp_tags) || (yytext[1]=='?' && short_tags)) {
//		zendlval->value.str.val = yytext; /* no copying - intentional */
//		zendlval->value.str.len = yyleng;
//		zendlval->type = IS_STRING;
		BEGIN(ST_IN_SCRIPTING);
		return T_OPEN_TAG_WITH_ECHO;
	} else {
//		zendlval->value.str.val = (char *) estrndup(yytext, yyleng);
//		zendlval->value.str.len = yyleng;
//		zendlval->type = IS_STRING;
		yylval.constant_val = new Constant(yytext);
		return T_INLINE_HTML;
	}
}

<INITIAL>"<%rules" {
	HANDLE_NEWLINE(yytext[yyleng-1]);
	BEGIN(ST_IN_SCRIPTING);
	return T_RULE_OPEN_TAG;
}

<INITIAL>"<%normalizers" {
	HANDLE_NEWLINE(yytext[yyleng-1]);
	BEGIN(ST_IN_SCRIPTING);
	return T_NORMALIZER_OPEN_TAG;
}


<INITIAL>"<%" {
	if (asp_tags) {
//		zendlval->value.str.val = yytext; /* no copying - intentional */
//		zendlval->value.str.len = yyleng;
//		zendlval->type = IS_STRING;
		BEGIN(ST_IN_SCRIPTING);
		return T_OPEN_TAG;
	} else {
//		zendlval->value.str.val = (char *) estrndup(yytext, yyleng);
//		zendlval->value.str.len = yyleng;
//		zendlval->type = IS_STRING;
		yylval.constant_val = new Constant(yytext);
		return T_INLINE_HTML;
	}
}


<INITIAL>"<?php"([ \t]|{NEWLINE}) {
	HANDLE_NEWLINE(yytext[yyleng-1]);
	BEGIN(ST_IN_SCRIPTING);
	return T_OPEN_TAG;
}


<INITIAL>"<?php_track_vars?>"{NEWLINE}? {
	warning("&lt;?php_track_vars?&gt; is no longer supported - please use the track_vars INI directive instead",yytext);
	HANDLE_NEWLINE(yytext[yyleng-1]);
//	zendlval->value.str.val = (char *) estrndup(yytext, yyleng);
//	zendlval->value.str.len = yyleng;
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_INLINE_HTML;
}

<ST_IN_SCRIPTING,ST_DOUBLE_QUOTES,ST_HEREDOC,ST_BACKQUOTE>"$"{LABEL} {
//	zend_copy_value(zendlval, (yytext+1), (yyleng-1));
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_VARIABLE;
}

<ST_IN_SCRIPTING,ST_DOUBLE_QUOTES,ST_HEREDOC,ST_BACKQUOTE>"$"{LNUM} {
       if (parseRules == ParseRules){
	  yylval.phpx_variable_val = new Variable(yytext+1);
	  return T_PHPX_VARIABLE;
       } else REJECT;
}

<ST_IN_SCRIPTING,ST_DOUBLE_QUOTES,ST_HEREDOC,ST_BACKQUOTE>"$*"{LNUM} {
       if (parseRules == ParseRules){
	  yylval.phpx_variable_val = new Variable(yytext+2);
	  return T_PHPX_LIST_VARIABLE;
       } else REJECT;
}

<ST_DOUBLE_QUOTES,ST_BACKQUOTE>{ENCAPSED_STRING_WITH_NEWLINE} {
	HANDLE_NEWLINES(yytext, yyleng);
//	zend_copy_value(zendlval, yytext, yyleng);
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_STRING;
}

<ST_IN_SCRIPTING>{LABEL} {
// 	zend_copy_value(zendlval, yytext, yyleng);
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_STRING;
}


<ST_HEREDOC>{ENCAPSED_STRING} {
 //	zend_copy_value(zendlval, yytext, yyleng);
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_STRING;
}


<ST_IN_SCRIPTING>{WHITESPACE} {
//	zendlval->value.str.val = yytext; /* no copying - intentional */
//	zendlval->value.str.len = yyleng;
//	zendlval->type = IS_STRING;
	HANDLE_NEWLINES(yytext, yyleng);
//	return T_WHITESPACE;
}


<ST_IN_SCRIPTING>"#"|"//" {
	BEGIN(ST_ONE_LINE_COMMENT);
	yymore();
}

<ST_ONE_LINE_COMMENT>"?"|"%"|">" {
	yymore();
}

<ST_ONE_LINE_COMMENT>[^\n\r?%>]+ {
	yymore();
}

<ST_ONE_LINE_COMMENT>{NEWLINE} {
//	zendlval->value.str.val = yytext; /* no copying - intentional */
//	zendlval->value.str.len = yyleng;
//	zendlval->type = IS_STRING;
	BEGIN(ST_IN_SCRIPTING);
	yylineno++;
//	yylval.constant_val = new Constant(yytext);
//	return T_COMMENT;
}

<ST_ONE_LINE_COMMENT>"?>"|"%>" {
    if (asp_tags || yytext[yyleng-2] != '%') { /* asp comment? */
//		zendlval->value.str.val = yytext; /* no copying - intentional */
//		zendlval->value.str.len = yyleng;
//		zendlval->type = IS_STRING;
		yyless(yyleng-2);
		BEGIN(ST_IN_SCRIPTING);
//		yylval.constant_val = new Constant(yytext);
//		return T_COMMENT;
	} else {
		yymore();
	}
}

<ST_IN_SCRIPTING>"/*" {
//	CG(comment_start_line) = CG(zend_lineno);
	BEGIN(ST_COMMENT);
	yymore();
}


<ST_COMMENT>[^*]+ {
	yymore();
}

<ST_COMMENT>"*/" {
	HANDLE_NEWLINES(yytext, yyleng);
	BEGIN(ST_IN_SCRIPTING);
//	yylval.constant_val = new Constant(yytext);
//	return T_COMMENT;
}

<ST_COMMENT>"*" {
	yymore();
}

<ST_IN_SCRIPTING>(("?>"|"</script"{WHITESPACE}*">"){NEWLINE}?) {
	BEGIN(INITIAL);
	
	return T_CLOSE_TAG;  /* implicit ';' at php-end tag */
}

<ST_IN_SCRIPTING>({WHITESPACE}|{NEWLINE})*"%>"{NEWLINE}? {
	if (asp_tags) {
		BEGIN(INITIAL);
//		zendlval->value.str.len = yyleng;
//		zendlval->type = IS_STRING;
//		zendlval->value.str.val = yytext; /* no copying - intentional */
		return T_CLOSE_TAG;  /* implicit ';' at php-end tag */
	} else {
		yyless(1);
		return yytext[0];
	}
}


<ST_IN_SCRIPTING>(["]([^$"\\]|("\\".))*["]) { //TODO: fixme
//	register char *s, *t;
//	char *end;
	
//	zendlval->value.str.val = estrndup(yytext+1, yyleng-2);
//	zendlval->value.str.len = yyleng-2;
//	zendlval->type = IS_STRING;
	HANDLE_NEWLINES(yytext, yyleng);
	
	/* convert escape sequences */
	/*
	s = t = zendlval->value.str.val;
	end = s+zendlval->value.str.len;
	while (s<end) {
		if (*s=='\\') {
			s++;
			if (s>=end) {
				continue;
			}
			switch(*s) {
				case 'n':
					*t++ = '\n';
					zendlval->value.str.len--;
					break;
				case 'r':
					*t++ = '\r';
					zendlval->value.str.len--;
					break;
				case 't':
					*t++ = '\t';
					zendlval->value.str.len--;
					break;
				case '\\':
				case '$':
				case '"':
					*t++ = *s;
					zendlval->value.str.len--;
					break;
				default:
					// check for an octal 
					if (ZEND_IS_OCT(*s)) {
						char octal_buf[4] = { 0, 0, 0, 0 };

						octal_buf[0] = *s;
						zendlval->value.str.len--;
						if ((s+1)<end && ZEND_IS_OCT(*(s+1))) {
							octal_buf[1] = *(++s);
							zendlval->value.str.len--;
							if ((s+1)<end && ZEND_IS_OCT(*(s+1))) {
								octal_buf[2] = *(++s);
								zendlval->value.str.len--;
							}
						}
						*t++ = (char) strtol(octal_buf, NULL, 8);
					} else if (*s=='x' && (s+1)<end && ZEND_IS_HEX(*(s+1))) {
						char hex_buf[3] = { 0, 0, 0};

						zendlval->value.str.len--; // for the 'x' 

						hex_buf[0] = *(++s);
						zendlval->value.str.len--;
						if ((s+1)<end && ZEND_IS_HEX(*(s+1))) {
							hex_buf[1] = *(++s);
							zendlval->value.str.len--;
						}
						*t++ = (char) strtol(hex_buf, NULL, 16);
					} else {
						*t++ = '\\';
						*t++ = *s;
					}
					break;
			}
			s++;
		} else {
			*t++ = *s++;
		}
	}
	*t = 0;
*/
	if (parseRules == ParseRules){
	   REJECT;
	}else{
		yytext[yyleng-1] = 0;
		yylval.constant_val = new Constant(yytext+1);
		return T_CONSTANT_ENCAPSED_STRING;
	}
}
<ST_IN_SCRIPTING>([']${LNUM}[']) {
	if (parseRules == ParseRules){
	  yytext[yyleng-1] = 0;
	  yylval.phpx_variable_val = new Variable(yytext+2);
	  return T_CONSTANT_ENCAPSED_PHPX_VARIABLE;
	}else{
	  REJECT;
	};
};

<ST_IN_SCRIPTING>([']([^'\\]|("\\".))*[']) { // TODO: fixme
//	register char *s, *t;
//	char *end;
	
//	zendlval->value.str.val = estrndup(yytext+1, yyleng-2);
//	zendlval->value.str.len = yyleng-2;
//	zendlval->type = IS_STRING;
	HANDLE_NEWLINES(yytext, yyleng);
	
	/* convert escape sequences */
	/*
	s = t = zendlval->value.str.val;
	end = s+zendlval->value.str.len;
	while (s<end) {
		if (*s=='\\') {
			s++;
			if (s>=end) {
				continue;
			}
			switch(*s) {
				case '\\':
				case '\'':
					*t++ = *s;
					zendlval->value.str.len--;
					break;
				default:
					*t++ = '\\';
					*t++ = *s;
					break;
			}
			s++;
		} else {
			*t++ = *s++;
		}
	}
	*t = 0;
	*/
	yytext[yyleng-1] = 0;
	yylval.constant_val = new Constant(yytext+1);
	return T_CONSTANT_ENCAPSED_STRING;
}


<ST_IN_SCRIPTING>["] {
	BEGIN(ST_DOUBLE_QUOTES);
	return '\"';
}

<ST_IN_SCRIPTING>"<<<"{TABS_AND_SPACES}{LABEL}{NEWLINE} {
	char *s;
	yylineno++;
	
	heredoc_len = yyleng-3-1-(yytext[yyleng-2]=='\r'?1:0);
	s = yytext+3;
	while ((*s == ' ') || (*s == '\t')) {
		s++;
		heredoc_len--;
	}
	heredoc = strndup(s, heredoc_len);
	BEGIN(ST_HEREDOC);
	return T_START_HEREDOC;
}
	

<ST_IN_SCRIPTING>[`] {
	BEGIN(ST_BACKQUOTE);
	return '`';
}

<ST_HEREDOC>^{LABEL}(";")?{NEWLINE} {
	int label_len;
	unsigned char unput_semicolon;

	yylineno++;
	if (yytext[yyleng-2]=='\r') {
		label_len = yyleng-2;
	} else {
		label_len = yyleng-1;
	}
	
	if (yytext[label_len-1]==';') {
		label_len--;
		unput_semicolon=1;
	} else{
		unput_semicolon=0;
	}

	if (label_len==heredoc_len && !memcmp(yytext, heredoc, label_len)) {
//		zendlval->value.str.val = estrndup(yytext, yyleng); /* unput destroys yytext */
//		zendlval->value.str.len = yyleng;
		if (unput_semicolon) {
			unput(';');
		}
		free(heredoc);
		heredoc=NULL;
		heredoc_len=0;
		BEGIN(ST_IN_SCRIPTING);
		return T_END_HEREDOC;
	} else {
// 		zend_copy_value(zendlval, yytext, yyleng);
//		zendlval->type = IS_STRING;
		yylval.constant_val = new Constant(yytext);
		return T_STRING;
	}
}


<ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>{ESCAPED_AND_WHITESPACE} {
	HANDLE_NEWLINES(yytext, yyleng);
//	zendlval->value.str.val = (char *) estrndup(yytext, yyleng);
//	zendlval->value.str.len = yyleng;
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_ENCAPSED_AND_WHITESPACE;
}

<ST_SINGLE_QUOTE>([^'\\]|\\[^'\\])+ {
	HANDLE_NEWLINES(yytext, yyleng);
 //	zend_copy_value(zendlval, yytext, yyleng);
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_ENCAPSED_AND_WHITESPACE;
}


<ST_DOUBLE_QUOTES>[`]+ {
// 	zend_copy_value(zendlval, yytext, yyleng);
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_ENCAPSED_AND_WHITESPACE;
}


<ST_BACKQUOTE>["]+ {
 //	zend_copy_value(zendlval, yytext, yyleng);
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_ENCAPSED_AND_WHITESPACE;
}


<ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>"$"[^a-zA-Z_\x7f-\xff{] {
//	zendlval->value.lval = (long) yytext[0];
	if (yyleng == 2) {
		yyless(1);
	}
	yylval.constant_val = new Constant(yytext);
	return T_CHARACTER;	
}


<ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>{ENCAPSED_TOKENS} {
//	zendlval->value.lval = (long) yytext[0];
	return yytext[0];
}

<ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>"{$" {
//	zendlval->value.lval = (long) yytext[0];
	yy_push_state(ST_IN_SCRIPTING);
	yyless(1);
	return T_CURLY_OPEN;
}


<ST_SINGLE_QUOTE>"\\'" {
//	zendlval->value.lval = (long) '\'';
	yylval.constant_val = new Constant(yytext);
	return T_CHARACTER;
}

<ST_SINGLE_QUOTE>"\\\\" {
//	zendlval->value.lval = (long)'\\';
	yylval.constant_val = new Constant(yytext);
	return T_CHARACTER;
}

<ST_DOUBLE_QUOTES>"\\\"" {
//	zendlval->value.lval = (long) '"';
	yylval.constant_val = new Constant(yytext);
	return T_CHARACTER;
}

<ST_BACKQUOTE>"\\`" {
//	zendlval->value.lval = (long) '`';
	yylval.constant_val = new Constant(yytext);
	return T_CHARACTER;
}

<ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>"\\"[0-7]{1,3} {
//	zendlval->value.lval = strtol(yytext+1, NULL, 8);
	yylval.constant_val = new Constant(yytext);
	return T_CHARACTER;
}

<ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>"\\x"[0-9A-Fa-f]{1,2} {
//	zendlval->value.lval = strtol (yytext+2, NULL, 16);
	yylval.constant_val = new Constant(yytext);
	return T_CHARACTER;
}

<ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_HEREDOC>"\\"{ANY_CHAR} {
	switch (yytext[1]) {
		case 'n':
//			zendlval->value.lval = (long) '\n';
			break;
		case 't':
//			zendlval->value.lval = (long) '\t';
			break;
		case 'r':
//			zendlval->value.lval = (long) '\r';
			break;
		case '\\':
//			zendlval->value.lval = (long) '\\';
			break;
		case '$':
//			zendlval->value.lval = (long) yytext[1];
			break;
		case '{':
//			zendlval->value.lval = (long) yytext[1];
			break;
		default:
//			zendlval->value.str.val = estrndup(yytext, yyleng);
//			zendlval->value.str.len = yyleng;
//			zendlval->type = IS_STRING;
			yylval.constant_val = new Constant(yytext);
			return T_BAD_CHARACTER;
			break;
	}
	yylval.constant_val = new Constant(yytext);
	return T_CHARACTER;
}


<ST_HEREDOC>["'`]+ {
//	zendlval->value.str.val = (char *) estrndup(yytext, yyleng);
//	zendlval->value.str.len = yyleng;
//	zendlval->type = IS_STRING;
	yylval.constant_val = new Constant(yytext);
	return T_ENCAPSED_AND_WHITESPACE;
}


<ST_DOUBLE_QUOTES>["] {
	BEGIN(ST_IN_SCRIPTING);
	return '\"';
}


<ST_BACKQUOTE>[`] {
	BEGIN(ST_IN_SCRIPTING);
	return '`';
}


<ST_SINGLE_QUOTE>['] {
	BEGIN(ST_IN_SCRIPTING);
	return '\'';
}


<ST_DOUBLE_QUOTES,ST_BACKQUOTE,INITIAL,ST_IN_SCRIPTING,ST_LOOKING_FOR_PROPERTY><<EOF>> {
	return 0;
}

<ST_COMMENT><<EOF>> {
	error("Unterminated comment starting line", yytext);
	return 0;
}
	


<ST_IN_SCRIPTING,INITIAL,ST_DOUBLE_QUOTES,ST_BACKQUOTE,ST_SINGLE_QUOTE,ST_HEREDOC>{ANY_CHAR} {
	error("Unexpected character in input:", yytext);
}







