#ifndef _STREE_H_
#define _STREE_H_

#include <iostream>
#include <algorithm>

#include "streenamedlist.h"

#define CMainList "MainList"

#define CStatementList "StatementList"
#define CExpressionList "ExpressionList"
#define CDeclareList "DeclareList"
#define CArrayPairList "ArrayPairList"
#define CTemplateList "TemplateList"
#define CInlineHTMLList "InlineHTMLList"

#define CPHP "PHP"
#define CEchoPHP "EchoPHP"

#define CCurlyBraces "CurlyBraces"
#define CSmartyBraces "SmartyBraces"
#define CDoubleQuote "DoubleQuote"

#define CIf "If"
#define CFor "For"
#define CWhile "While"
#define CDo "Do"
#define CSwitch "Switch"
#define CBreak "Break"
#define CContinue "Continue"
#define CReturn "Return"
#define CGlobal "Global"
#define CStatic "Static"
#define CEcho "Echo"
#define CTemplate "Template"
#define CSmartyAssign "SmartyAssign"
#define CInlineHTML "InlineHTML"
#define CExpression "Expression"
#define CUnset "Unset"
#define CUse "Use"
#define CForeach "Foreach"
#define CDeclare "Declare"
#define CEmptyStatement "EmptyStatement"
#define CFunction "Function"
#define CClass "Class"

#define CCaseList "CaseList"
#define CEncapsList "EncapsList"
#define CAssignmentList "AssignmentList"

#define CConst "Const"
#define CCase "Case"
#define CParameter "Parameter"
#define CClassVariable "ClassVariable"
#define CUnaryOperator "UnaryOperator"
#define CPostUnaryOperator "PostUnaryOperator"
#define CBinaryOperator "BinaryOperator"
#define CAList "AList"
#define CEncapsExpr "EncapsExpr"
#define CAskOperator "AskOperator"
#define CExit "Exit"
#define CPrint "Print"
#define CArray "Array"
#define CFunctionCall "FunctionCall"
#define CArrayPair "ArrayPair"
#define CObjectOperator "ObjectOperator"
#define CArrayOperator "ArrayOperator"
#define CMapOperator "MapOperator"
#define OpRef "$"
#define CIsset "Isset"
#define CConstantEncapsedString "ConstantEncapsedString"
#define CPHPVariable "PHPVariable"
#define CConstantString "ConstantString"

/// statement list.
class StatementList : public NamedList{
 public:
  /// Constructor
  StatementList() ;
  /// Constructor
  StatementList(Node *statement) ;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// InlineHTML list.
class InlineHTMLList : public NamedList{
 public:
  /// Constructor
  InlineHTMLList() ;
  /// Constructor
  InlineHTMLList(Node *statement) ;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// Template list \sa ::ContextRule.
class TemplateList : public NamedList{
 public:
  /// Constructor
  TemplateList() ;
  /// Constructor
  TemplateList(Node *statement);

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// Expression list.
class ExpressionList : public NamedList{
 public:
  /// Constructor
  ExpressionList() ;
  /// Constructor
  ExpressionList(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Array pait list.
class ArrayPairList : public NamedList{
 public:
  /// Constructor
  ArrayPairList() ;
  /// Constructor
  ArrayPairList(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Declaration statements list.
class DeclareList : public NamedList{
 public:
  /// Constructor
  DeclareList() ;
  /// Constructor
  DeclareList(Node *statement) ;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// Case list.
class CaseList : public NamedList{
 public:
  /// Constructor
  CaseList() ;
  /// Constructor
  CaseList(Node *statement) ;

  virtual ostream& php(ostream &os) const;
    
  virtual Node* copy() const;
};

/// Encaps list -- for double quotes. 
class EncapsList : public NamedList{
 public:
  /// Constructor
  EncapsList() ;
  /// Constructor
  EncapsList(Node *statement) ;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

/// Assignment list.
class AssignmentList : public NamedList{
 public:
  /// Constructor
  AssignmentList() ;
  /// Constructor
  AssignmentList(Node *statement) ;

  virtual ostream& php(ostream &os) const;  
  virtual Node* copy() const;
};

/// Encapsulates PHP code.
class PHP : public Unary{
 public:
  /// Constructor
  PHP(Node *statement);

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

/// Encapslates PHP code with echo. 
class EchoPHP : public Unary{
 public:
  /// Constructor
  EchoPHP(Node *statement) ;

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

/// Encapsulates node with curly braces.
class CurlyBraces : public Unary{
 public:
  /// Constructor
  CurlyBraces(Node *statement) ;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// Encapsulates node with Smarty braces.
class SmartyBraces : public Unary{
 public:
  /// Constructor
  SmartyBraces(Node *statement) ;

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

///  Encapsulates node with double quotes.
class DoubleQuote : public Unary{
 public:
  /// Constructor
  DoubleQuote(Node *statement) ;

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

/// If statement.
/*!
The meaning of arguments is as follows:
  - first is a condition,
  - second is a conditonal statement,
  - third is a statment in else.
*/
class If : public Ternary{
 public:
  /// Constructor
  If(Node* statement1, Node* statement2, Node* statement3) ;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// For statement.
/*!
The meaning of arguments is as follows:
  - first three give expressions in for statment,
  - forth is a for statment.
*/
class For : public Quadary{
 public:
  /// Constructor
  For(Node* statement1, Node* statement2, Node* statement3, Node* statement4);

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// While statement.
/*!
The meaning of arguments is as follows:
  - first gives condition,
  - second is a while statement.
*/
class While : public Binary{
 public:
  /// Constructor
  While(Node* statement1, Node* statement2);

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

/// Do statement.
/*!
The meaning of arguments is as follows:
  - first gives condition,
  - second is a do statement.
*/
class Do : public Binary{
 public:
  /// Constructor
  Do(Node* statement1, Node* statement2) ;

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

/// Switch statement.
/*!
The meaning of arguments is as follows:
  - first is an expression,
  - second is a case list \sa CaseList.
*/
class Switch : public Binary{
 public:
  /// Constructor
  Switch(Node* statement1, Node* statement2) ;

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

/// Break statement.
/*!
The argumnets is a break expresson.
*/
class Break : public Unary{
 public:
  /// Constructor
  Break(Node *statement) ;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// Contiune statement.
/*!
The argumnets is a continue expresson.
*/
class Continue : public Unary{
 public:
  /// Constructor
  Continue(Node *statement) ;

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

/// Return statement.
/*!
The argumnets is a return expresson.
*/
class Return : public Unary{
 public:
  /// Constructor
  Return(Node *statement) ;

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

/// Global definition.
class Global : public Unary{
 public:
  /// Constructor
  Global(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Static definition.
class Static : public Unary{
 public:
  /// Constructor
  Static(Node *statement);

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

/// Echo.
class Echo : public Unary{
 public:
  /// Constructor
  Echo(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// A Template \sa TemplateList \sa ContextRule.
class Template : public Unary{
 public:
  /// Constructor
  Template(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Assignment to smarty variable. 
/*!
The meaning of arguments is as follows:
  - first is a variable,
  - second is a value assigned.
When writing by php SmartyAssign::phpOut is written.
*/
class SmartyAssign : public Binary{
  /// A php output string.
  string phpOut;
 public:
  /// Constructor
  SmartyAssign(Node* statement1, Node* statement2) ;
  /// Constructor
  SmartyAssign(Node* statement1, Node* statement2, string iphpOut) ;

  virtual ostream& php(ostream &os) const;

  /// Returns variable.
  const Node* getVariable() const;
  
  /// Return value assigned.
  const Node* getAssignment() const;

  /// Sets SmartyAssign::phpOut 
  void setPHPOut(string iphpOut);

  virtual Node* copy() const;
};

/// Inline HTML.
class InlineHTML : public Unary{
 public:
  /// Constructor
  InlineHTML(Node *statement) ;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;

  /// Returns HTML
  string getHTML() const;
  /// Sets HTML
  void setHTML(const string &html);
};

/// Expression
class Expression : public Unary{
 public:
  /// Constructor
  Expression(Node *statement) ;
  
  virtual ostream& php(ostream &os) const;
  
  virtual Node* copy() const;
};

/// Unset.
class Unset : public Unary{
 public:
  /// Constructor
  Unset(Node *statement) ;
  
  virtual ostream& php(ostream &os) const;
  
  virtual Node* copy() const;
};

/// Use.
class Use : public Unary{
 public:
  /// Constructor
  Use(Node *statement) ;
  
  virtual ostream& php(ostream &os) const; 

  virtual Node* copy() const;
};

/// Foreach statement
/*!
The meaning of arguments is as follows:
  - first is a variable,
  - second is an as variable,
  - third is an optional arg,
  - forth is a foreach statement.
*/
class Foreach : public Quadary{
 public:
  /// Constructor
  Foreach(Node* statement1, Node* statement2, Node* statement3, Node* statement4) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Declare statement
/*!
The meaning of arguments is as follows:
  - first is a declare list,
  - second is a declaration statement,
*/
class Declare : public Binary{
 public:
  /// Constructor
  Declare(Node* statement1, Node* statement2) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Empty statement
/*! In contrary to Pin it is a list.
 */
class EmptyStatement : public Nullary{
 public:
  /// Constructor
  EmptyStatement() ;

  virtual ostream& php(ostream &os) const;    
  virtual Node* copy() const;
};

/// Function definition
/*!
The meaning of arguments is as follows:
  - first -- can be Pin or "&" constant,
  - second is a function name,
  - third is a paramentr list,
  - forth is a function statement list.
*/
class Function : public Quadary{
 public:
  /// Constructor
  Function(Node* statement1, Node* statement2, Node* statement3, Node* statement4) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Class definition
/*!
The meaning of arguments is as follows:
  - first is a class name,
  - second is a class name that the class extends,
  - thirs is a class statement list.
*/
class Class : public Ternary{
 public:
  /// Constructor
  Class(Node* statement1, Node* statement2, Node* statement3) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// No idea what it is.
class Const : public Binary{
 public:
  /// Constructor
  Const(Node* statement1, Node* statement2) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Case statement \sa CaseList \sa Switch
class Case : public Binary{
 public:
  /// Constructor
  Case(Node* statement1, Node* statement2);

  virtual ostream& php(ostream &os) const;
  
  virtual Node* copy() const;
};

/// Paramenter or a paramenter definition \sa ParamenterList
/*!
The meaning of arguments is as follows:
  - first can be Pin, "&" or "const",
  - second is a paramenter name,
  - can be a parameter value.
*/
class Parameter : public Ternary{
 public:
  /// Constructor
  Parameter(Node* statement1, Node* statement2, Node* statement3) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Reference to a class
class ClassVariable : public Unary{
 public:
  /// Constructor
  ClassVariable(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Unary operatorc
/*!
  The name of the operator is given by \a oname.
 */
class UnaryOperator : public Binary{
 public:
  /// Constructor
  UnaryOperator(string oname, Node* statement2) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

/// Post unary operator.
/*!
  The name of the operator is given by \a oname.
 */
class PostUnaryOperator : public Binary{
 public:
  /// Constructor
  PostUnaryOperator(string oname, Node* statement2) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

/// Binary operator
/*!
  The name of the operator is given by \a oname.
 */
class BinaryOperator : public Ternary{
 public:
  /// Constructor
  BinaryOperator(string oname, Node* statement2, Node* statement3) ;
  BinaryOperator(Node* ioperator, Node* statement2, Node* statement3) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

/// List statement
class AList : public Unary{
 public:
  /// Constructor
  AList(Node *statement);

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Encapsed statement
class EncapsExpr : public Ternary{
 public:
  /// Constructor
  EncapsExpr(string ebegin, Node *statement, string eend) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

/// ? : Operator
class AskOperator : public Ternary{
 public:
  /// Constructor
  AskOperator(Node* statement1, Node* statement2, Node* statement3);

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

/// Exit statement
class Exit : public Unary{
 public:
  /// Constructor
  Exit(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Print statement
class Print : public Unary{
 public:
  /// Constructor
  Print(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Array statement
class Array : public Unary{
 public:
  /// Constructor
  Array(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Function call
/*!
The meaning of arguments is as follows:
  - first a namespace,
  - second is a function name,
  - can be a parameter list.
*/
class FunctionCall : public Ternary{
 public:
  /// Constructor
  FunctionCall(Node* statement1, Node* statement2, Node* statement3) ;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// Array pair \sa Array \sa ArrayPairList
/*!
The meaning of arguments is as follows:
  - first is an index or Pin,
  - second is a value.
*/
class ArrayPair : public Binary{
 public:
  /// Constructor
  ArrayPair(Node* statement1, Node* statement2) ;

  virtual ostream& php(ostream &os) const;
  
  virtual Node* copy() const;
};

/// Object operator ->
/*!
The meaning of arguments is as follows:
  - first is an object name,
  - second is an object  property.
*/
class ObjectOperator : public Binary{
 public:
  /// Constructor
  ObjectOperator(Node* statement1, Node* statement2) ;

  virtual ostream& php(ostream &os) const;
  
  virtual Node* copy() const;
};

/// Array operator []
/*!
The meaning of arguments is as follows:
  - first is an array name,
  - second is an array index.
*/
class ArrayOperator : public Binary{
 public:
  /// Constructor
  ArrayOperator(Node* statement1, Node* statement2) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Map operator -- no idea if correctly used
class MapOperator : public Binary{
 public:
  /// Constructor
  MapOperator(Node* statement1, Node* statement2) ;

  virtual ostream& php(ostream &os) const;
  
  virtual Node* copy() const;
};

/// Reference &
class Ref : public UnaryOperator{
 public:
  /// Constructor
  Ref(Node* statement) ;
};

/// Isset statement
class Isset : public Unary{
 public:
  /// Constructor
  Isset(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

/// Single quote encapsed string constant 
class ConstantEncapsedString : public Unary{
 public:
  /// Constructor
  ConstantEncapsedString(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

/// String constant 
class ConstantString : public Unary{
 public:
  /// Constructor
  ConstantString(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

/// PHPVariable statement
class PHPVariable : public Unary{
 public:
  /// Constructor
  PHPVariable(Node *statement) ;

  virtual ostream& php(ostream &os) const;

  virtual Node* copy() const;
};

#endif



