#ifndef _STREE_CONSTANT_H_
#define _STREE_CONSTANT_H_

#include <sstream>
#include "streenode.h"

#define CConstant "Constant"

/// A string constant
/*! Used for representing following php elements:
  - variables
  - string constants
  - number constants
  - inline HTML
*/

class Constant : public Node
{
  /// The value of the constant.
  string value;
 public:

  /// Creates Constant from a string.
  Constant(const string &ivalue, const string &icomment = "", const int ilineno = 0) : Node(icomment,ilineno), value(ivalue) {};
  
  virtual SType getType() const { return SConstant; };
  
  /// Returns the value of the constatnt. 
  string getValue() const { return value; };

  virtual string getName() const { return value; };
    
  virtual ostream& print(ostream &os) const;

  virtual ostream& php(ostream &os) const;

  virtual bool match(const Node *p, Assignment &assign, const NameSpace &nameSpace)const;

  virtual bool matchToBegining(const List *p, Assignment &assign, const NameSpace &nameSpace) const;

  virtual bool compare(const Node *p) const;
    
  virtual Node* copy() const;
  
  virtual void getVariables(VariableList &vlist) const;

  /// Tries to convert the value to number.
  int toInt() const;

  virtual Node* substitute(const Assignment &assign);

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

#endif
