#include "streelist.h"

List *List::copyContent(List *newList) const{
    const_iterator it = begin();
    const_iterator eit = end();
    while (it!=eit){
      newList->push_back((*it)->copy());
      it++;
    };
    return newList;
}

List::List(string iname) : name(iname) {};

List::List(string iname, Node *statement): name(iname){
    push_back(statement);
};

List* List::push_back(Node *statement){
    list<Node*>::push_back(statement);
    return this;
};

List* List::push_front(Node *statement){
    list<Node*>::push_front(statement);
    return this;
};

List* List::append(List* toappend){
    //merge(*toappend);
    insert(end(),toappend->begin(),toappend->end());
    toappend->clear();
    delete toappend;
    return this;
} 


ostream& List::print(ostream &os) const{
    os << name << "[";
    const_iterator it = begin();
    const_iterator eit = end();
    while (it!=eit){
      (*it)->print(os);
      it++;
      if (it!=eit) os << ":";
    };
    return os << "]";
};

ostream& List::php(ostream &os) const{
      cerr <<"ERROR: php can not be called for unnamed lists  : " << name << endl;
      return print(os);
}
  
bool List::match(const Node *p, Assignment &assign, const NameSpace &nameSpace) const{
    // todo : bosuga SListVariable
    if (p->getType()!=SList) return false;
    
    const List *lp = dynamic_cast<const List*>(p);
    if ((lp->getLength()!=getLength())||(lp->getName()!=getName())) return false;

    const_iterator it = begin();
    const_iterator eit = end();
    const_iterator pit = lp->begin();
    //    const_iterator peit = lp->end();
    while (it!=eit){
      if (!(*it)->match(*pit,assign,nameSpace)) return false;
      it++;
      pit++;
    };
    return true;
};

bool List::matchToBegining(const List *p, Assignment &assign, const NameSpace &nameSpace) const{
    if (p->getLength()<getLength()) return false;

    const_iterator pit = p->begin();
    //    const_iterator peit = p->end();
    const_iterator it = begin();
    const_iterator eit = end();

    while (it!=eit){
      if (!(*it)->match(*pit,assign,nameSpace)) return false;
      it++;
      pit++;
    };
    return true;
};


bool List::compare(const Node *p) const{
    if (p->getType()!=SList) return false;

    List *lp = (List*)p;
    if ((lp->getLength()!=getLength())||(lp->getName()!=getName())) return false;

    const_iterator it = begin();
    const_iterator eit = end();
    const_iterator pit = lp->begin();
    //    const_iterator peit = lp->end();
    while (it!=eit){
      if (!(*it)->compare(*pit)) return false;
      it++;
      pit++;
    }
    return true;
};

Node* List::substitute(const Assignment &assign){
    iterator it = begin();
    iterator eit = end();
    while (it!=eit){
      substituteNode(*it,assign);
      it++;
    };
    return this;
};

Node* List::copy() const{
    return copyContent(new List(getName()));
};

List* List::copyList() const{
    return dynamic_cast<List*>(copy());
};

List::~List(){
    iterator it = begin();
    iterator eit = end();
    while (it!=eit){
      delete *it;
      it++;
    };    
};

void List::getVariables(VariableList &vlist) const{
    const_iterator it = begin();
    const_iterator eit = end();
    while (it!=eit){
      (*it)->getVariables(vlist);
      it++;
    };    
};

Node* List::transform(const Transformer* transformer){
  iterator it = begin();
  iterator eit = end();
  while (it!=eit){
    Node *targ = (*it)->transform(transformer);
    if (targ!=*it) {
      delete *it;
      *it = targ;
    }
    it++;
  };
  return transformer->transform(this);
};

bool List::forall(const Tester* tester, const NameSpace &nameSpace) const{
  if (!tester->test(this, nameSpace))
    return false;
  const_iterator it = begin();
  const_iterator eit = end();
  while (it!=eit){
    if (!(*it)->forall(tester, nameSpace))
      return false;
    it++;
  };
  return true;
};

bool List::exists(const Tester* tester, const NameSpace &nameSpace) const{
  if (tester->test(this, nameSpace))
    return true;
  const_iterator it = begin();
  const_iterator eit = end();
  while (it!=eit){
    if ((*it)->exists(tester, nameSpace))
      return true;
    it++;
  };
  return false;
};

string List::evaluate(const NameSpace &nameSpace) const{
  throw "ERROR: List::evaluate";
};
