#ifndef _STREE_LIST_H_
#define _STREE_LIST_H_

#include <list>

#include "streenode.h"

/// List is used for representing multiargument php elements.

class List : public Node, public list<Node*>
{
 protected:
  /// Name of the list. 
  string name;

  /// Copies content of the list to \a newList nad returns pointer to \a newList. 
  List *copyContent(List *newList) const;

 public:

  virtual SType getType() const { return SList; };
  virtual string getName() const { return name; };

  /// Returns the length of the list. 
  virtual int getLength() const { return size(); };

  /// Creates a list with a given name. 
  List(string iname);

  /// Creates a list with a given name containing one elemnet.
  List(string iname, Node *statement);

  /// Pushes an element to the end of the list. 
  virtual List* push_back(Node *statement);

  /// Pushes an element to the front of the list. 
  virtual List* push_front(Node *statement);

  /// Append \a toappend to the list.
  List* append(List* toappend);

  virtual ostream& print(ostream &os) const;

  virtual ostream& php(ostream &os) const;

  virtual bool match(const Node *p, Assignment &assign, const NameSpace &nameSpace) const;

  virtual bool matchToBegining(const List *p, Assignment &assign, const NameSpace &nameSpace) const;

  virtual bool compare(const Node *p) const;

  virtual Node* copy() const;

  /// Copies the list.
  List* copyList() const;
  
  /// Destroys the list.
  virtual ~List();

  virtual void getVariables(VariableList &vlist) const;

  virtual Node* transform(const Transformer* transformer);

  virtual bool forall(const Tester* tester, const NameSpace &nameSpace) const;

  virtual bool exists(const Tester* tester, const NameSpace &nameSpace) const;

  virtual Node* substitute(const Assignment &assign);

  virtual string evaluate(const NameSpace &nameSpace) const;  
};

#endif

