#ifndef _STREE_NAMED_LIST_H_
#define _STREE_NAMED_LIST_H_

#include "streelist.h"

/// A list that must have a name \sa List.
class NamedList : public List{
 public:
  /// Creates a list with given name.
  NamedList(string name);
  /// Creates a list with given name and first element/
  NamedList(string name,Node *statement);
};


/// A node without childrens. 
class Nullary : public List{
 public:
  /// Creates Nullary element with given name. 
  Nullary(string name);

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// A node with one child.
class Unary : public List{
 public:
  /// Creates Unary with given name and child. 
  Unary(string name,Node* statement);

  /// Returns the child of the node.
  Node* getStatement1();
  /// Returns the child of the node.
  const Node* getStatement1() const;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// A node with two children.
class Binary : public List{
 public:

  /// Creates Binary with given name and children. 
  Binary(string name,Node* statement1, Node* statement2) ;
  
  /// Returns the first child of the node.
  Node* getStatement1() ;
  /// Returns the second child of the node.
  Node* getStatement2() ;
  
  /// Returns the first child of the node.
  const Node* getStatement1() const;
  /// Returns the second child of the node.
  const Node* getStatement2() const;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;

};

/// A node with three children.
class Ternary : public List{
 public:

  /// Creates Ternary with given name and children. 
  Ternary(string name,Node* statement1, Node* statement2, Node* statement3) ;
  
  /// Returns the first child of the node.
  Node* getStatement1() ;
  /// Returns the second child of the node.
  Node* getStatement2() ;
  /// Returns the third child of the node.
  Node* getStatement3() ;
  
  /// Returns the first child of the node.
  const Node* getStatement1() const;
  /// Returns the second child of the node.
  const Node* getStatement2() const;
  /// Returns the third child of the node.
  const Node* getStatement3() const;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

/// A node with four children.
class Quadary : public List{
 public:
  /// Creates Quadary with given name and children. 
  Quadary(string name,Node* statement1, Node* statement2, Node* statement3, Node* statement4) ;
  
  /// Returns the first child of the node.
  Node* getStatement1();
  /// Returns the second child of the node.
  Node* getStatement2();
  /// Returns the third child of the node.
  Node* getStatement3();
  /// Returns the forth child of the node.
  Node* getStatement4();

  /// Returns the first child of the node.  
  const Node* getStatement1() const;
  /// Returns the second child of the node.
  const Node* getStatement2() const;
  /// Returns the third child of the node.
  const Node* getStatement3() const;
  /// Returns the forth child of the node.
  const Node* getStatement4() const;

  virtual ostream& php(ostream &os) const;  

  virtual Node* copy() const;
};

#endif
