#include "streevariable.h"
#include "streelist.h"

Variable::Variable(int iid) : id(iid) {};

Variable::Variable(Constant *name) : id(name->toInt()){
      delete name;
};

Variable::Variable(const string &name) {
      istringstream is(name.c_str());
      is >> id;
};

ostream& Variable::print(ostream &os) const{
      return os << id;
};

ostream& Variable::php(ostream &os) const{
      //      cerr <<"ERROR: php can not be called for Variables" << endl;
      return os << "$" << id;
}

    
bool Variable::match(const Node *p, Assignment &assign, const NameSpace &nameSpace) const{
  if (assign.find(getId())!=assign.end())
    return assign[getId()]->match(p,assign,nameSpace);
  
  assign[getId()] = p->copy();
  return true;
};

bool Variable::matchToBegining(const List *p, Assignment &assign, const NameSpace &nameSpace) const{
  if (assign.find(getId())!=assign.end())
    return assign[getId()]->match(p,assign,nameSpace);  
  assign[getId()] = p->copy();
  return true;
};

bool Variable::compare(const Node *p) const{
      if (p->getType()!=SVariable) return false;      
      if (((Variable*)p)->getId()!=getId()) return false;
      return true;
};
    
Node* Variable::substitute(const Assignment &assign){
      if (assign.find(getId())!=assign.end()){
        return assign.find(getId())->second->copy();
      }
      return this;
};
    
Node* Variable::copy() const{
      return new Variable(getId());
};

void Variable::getVariables(VariableList &vlist) const{
    vlist.insert(getId());
};

string Variable::evaluate(const NameSpace &nameSpace) const{
  throw "ERROR: Variable::evaluate";
};
