#ifndef _TRANSFORMERSET_H_
#define _TRANSFORMERSET_H_

#include <list>
#include "streenode.h"
#include "echonormalizer.h"
#include "tplconstantnormalizer.h"

/// A set of transformers
class TransformerSet : public list<Node::Transformer*> {
 public:
  TransformerSet(Node::Transformer **b, Node::Transformer **e) : list<Node::Transformer*>(b,e) {};

  void apply(Node *inNode);
  ~TransformerSet();
};

static Node::Transformer* phpPreNormalizersVals[] = { new EchoNormalizer() };
static Node::Transformer* phpPostNormalizersVals[] = {};
static Node::Transformer* tplPostNormalizersVals[] = { new TPLConstantNormalizer() };

static const size_t phpPreNormalizersSize = sizeof phpPreNormalizersVals / sizeof phpPreNormalizersVals[0];
/// Pre normalizing transformers for php
static TransformerSet phpPreNormalizers(phpPreNormalizersVals, phpPreNormalizersVals + phpPreNormalizersSize);

static const size_t phpPostNormalizersSize = sizeof phpPostNormalizersVals / sizeof phpPostNormalizersVals[0];
/// Post normalizing transformers for php
static TransformerSet phpPostNormalizers(phpPostNormalizersVals, phpPostNormalizersVals + phpPostNormalizersSize);

static const size_t tplPostNormalizersSize = sizeof tplPostNormalizersVals / sizeof tplPostNormalizersVals[0];
/// Post normalizing transformers for tpl
static TransformerSet tplPostNormalizers(tplPostNormalizersVals, tplPostNormalizersVals + tplPostNormalizersSize);

#endif
